/****************************************************************************
** Meta object code from reading C++ file 'part.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/part/part.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'part.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Okular4PartE_t {};
} // unnamed namespace

template <> constexpr inline auto Okular::Part::qt_create_metaobjectdata<qt_meta_tag_ZN6Okular4PartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Okular::Part",
        "D-Bus Interface",
        "org.kde.okular",
        "enablePrintAction",
        "",
        "enable",
        "openSourceReference",
        "absFileName",
        "line",
        "column",
        "viewerMenuStateChange",
        "enabled",
        "enableCloseAction",
        "mimeTypeChanged",
        "QMimeType",
        "mimeType",
        "urlsDropped",
        "QList<QUrl>",
        "urls",
        "fitWindowToPage",
        "QSize",
        "pageViewPortSize",
        "pageSize",
        "maxRecentItemsChanged",
        "requestOpenNewlySignedFile",
        "path",
        "pageNumber",
        "goToPage",
        "Q_NOREPLY",
        "page",
        "openDocument",
        "doc",
        "pages",
        "currentPage",
        "currentDocument",
        "documentMetaData",
        "metaData",
        "slotPreferences",
        "slotFind",
        "slotPrintPreview",
        "slotPreviousPage",
        "slotNextPage",
        "slotGotoFirst",
        "slotGotoLast",
        "slotTogglePresentation",
        "slotToggleChangeColors",
        "slotSetChangeColors",
        "active",
        "reload",
        "enableStartWithPrint",
        "enableExitAfterPrint",
        "enableStartWithFind",
        "text",
        "setEditorCmd",
        "editorCmd",
        "slotOpenContainingFolder",
        "slotCopyAnnotation",
        "slotCopyTextSelectionOrAnnotation",
        "slotPasteAnnotation",
        "openUrlFromDocument",
        "QUrl",
        "url",
        "openUrlFromBookmarks",
        "handleDroppedUrls",
        "slotGoToPage",
        "slotHistoryBack",
        "slotHistoryNext",
        "slotAddBookmark",
        "slotRenameBookmarkFromMenu",
        "slotRemoveBookmarkFromMenu",
        "slotRenameCurrentViewportBookmark",
        "slotPreviousBookmark",
        "slotNextBookmark",
        "slotFindNext",
        "slotFindPrev",
        "slotSaveFileAs",
        "showOkularArchiveAsDefaultFormat",
        "slotNewConfig",
        "slotShowMenu",
        "const Okular::Page*",
        "QPoint",
        "point",
        "slotShowTOCMenu",
        "Okular::DocumentViewport",
        "vp",
        "title",
        "slotShowProperties",
        "slotShowEmbeddedFiles",
        "slotShowLeftPanel",
        "slotShowBottomBar",
        "slotShowPresentation",
        "slotHidePresentation",
        "slotUpdateHamburgerMenu",
        "slotExportAs",
        "QAction*",
        "slotImportPSFile",
        "slotAboutBackend",
        "slotReload",
        "close",
        "cannotQuit",
        "slotShowFindBar",
        "slotHideFindBar",
        "slotJobStarted",
        "KIO::Job*",
        "job",
        "slotJobFinished",
        "KJob*",
        "loadCancelled",
        "reason",
        "setWindowTitleFromDocument",
        "updateViewActions",
        "updateBookmarksActions",
        "enableTOC",
        "slotRebuildBookmarkMenu",
        "enableLayers",
        "enableSidebarSignaturesItem",
        "saveFile",
        "slotPrint",
        "slotFileDirty",
        "slotAttemptReload",
        "oneShot",
        "newUrl",
        "psTransformEnded",
        "QProcess::ExitStatus",
        "errorMessage",
        "message",
        "duration",
        "warningMessage",
        "noticeMessage",
        "moveSplitter",
        "sideWidgetSize",
        "finishSigning",
        "slotAccessibilityPreferences",
        "slotAnnotationPreferences",
        "slotHandleActivatedSourceReference",
        "col",
        "bool*",
        "handled",
        "activateTabIfAlreadyOpenFile"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'enablePrintAction'
        QtMocHelpers::SignalData<void(bool)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'openSourceReference'
        QtMocHelpers::SignalData<void(const QString &, int, int)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 8 }, { QMetaType::Int, 9 },
        }}),
        // Signal 'viewerMenuStateChange'
        QtMocHelpers::SignalData<void(bool)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'enableCloseAction'
        QtMocHelpers::SignalData<void(bool)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'mimeTypeChanged'
        QtMocHelpers::SignalData<void(const QMimeType &)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'urlsDropped'
        QtMocHelpers::SignalData<void(const QList<QUrl> &)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'fitWindowToPage'
        QtMocHelpers::SignalData<void(const QSize, const QSize)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 20, 22 },
        }}),
        // Signal 'maxRecentItemsChanged'
        QtMocHelpers::SignalData<void(const int)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Signal 'requestOpenNewlySignedFile'
        QtMocHelpers::SignalData<void(const QString &, int)>(24, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 }, { QMetaType::Int, 26 },
        }}),
        // Slot 'goToPage'
        QtMocHelpers::SlotData<void(uint)>(27, 28, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::UInt, 29 },
        }}),
        // Slot 'openDocument'
        QtMocHelpers::SlotData<void(const QString &)>(30, 28, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 31 },
        }}),
        // Slot 'pages'
        QtMocHelpers::SlotData<uint()>(32, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::UInt),
        // Slot 'currentPage'
        QtMocHelpers::SlotData<uint()>(33, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::UInt),
        // Slot 'currentDocument'
        QtMocHelpers::SlotData<QString()>(34, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'documentMetaData'
        QtMocHelpers::SlotData<QString(const QString &) const>(35, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString, {{
            { QMetaType::QString, 36 },
        }}),
        // Slot 'slotPreferences'
        QtMocHelpers::SlotData<void()>(37, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotFind'
        QtMocHelpers::SlotData<void()>(38, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotPrintPreview'
        QtMocHelpers::SlotData<void()>(39, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotPreviousPage'
        QtMocHelpers::SlotData<void()>(40, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotNextPage'
        QtMocHelpers::SlotData<void()>(41, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotGotoFirst'
        QtMocHelpers::SlotData<void()>(42, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotGotoLast'
        QtMocHelpers::SlotData<void()>(43, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotTogglePresentation'
        QtMocHelpers::SlotData<void()>(44, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotToggleChangeColors'
        QtMocHelpers::SlotData<void()>(45, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotSetChangeColors'
        QtMocHelpers::SlotData<void(bool)>(46, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Bool, 47 },
        }}),
        // Slot 'reload'
        QtMocHelpers::SlotData<void()>(48, 28, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'enableStartWithPrint'
        QtMocHelpers::SlotData<void()>(49, 28, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'enableExitAfterPrint'
        QtMocHelpers::SlotData<void()>(50, 28, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'enableStartWithFind'
        QtMocHelpers::SlotData<void(const QString &)>(51, 28, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 52 },
        }}),
        // Slot 'setEditorCmd'
        QtMocHelpers::SlotData<void(const QString &)>(53, 28, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 54 },
        }}),
        // Slot 'slotOpenContainingFolder'
        QtMocHelpers::SlotData<void()>(55, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotCopyAnnotation'
        QtMocHelpers::SlotData<void()>(56, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotCopyTextSelectionOrAnnotation'
        QtMocHelpers::SlotData<void()>(57, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'slotPasteAnnotation'
        QtMocHelpers::SlotData<void()>(58, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'openUrlFromDocument'
        QtMocHelpers::SlotData<void(const QUrl &)>(59, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 60, 61 },
        }}),
        // Slot 'openUrlFromBookmarks'
        QtMocHelpers::SlotData<void(const QUrl &)>(62, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 60, 61 },
        }}),
        // Slot 'handleDroppedUrls'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(63, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'slotGoToPage'
        QtMocHelpers::SlotData<void()>(64, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHistoryBack'
        QtMocHelpers::SlotData<void()>(65, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHistoryNext'
        QtMocHelpers::SlotData<void()>(66, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAddBookmark'
        QtMocHelpers::SlotData<void()>(67, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRenameBookmarkFromMenu'
        QtMocHelpers::SlotData<void()>(68, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRemoveBookmarkFromMenu'
        QtMocHelpers::SlotData<void()>(69, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRenameCurrentViewportBookmark'
        QtMocHelpers::SlotData<void()>(70, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPreviousBookmark'
        QtMocHelpers::SlotData<void()>(71, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNextBookmark'
        QtMocHelpers::SlotData<void()>(72, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFindNext'
        QtMocHelpers::SlotData<void()>(73, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFindPrev'
        QtMocHelpers::SlotData<void()>(74, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSaveFileAs'
        QtMocHelpers::SlotData<bool(bool)>(75, 4, QMC::AccessProtected, QMetaType::Bool, {{
            { QMetaType::Bool, 76 },
        }}),
        // Slot 'slotSaveFileAs'
        QtMocHelpers::SlotData<bool()>(75, 4, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'slotNewConfig'
        QtMocHelpers::SlotData<void()>(77, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowMenu'
        QtMocHelpers::SlotData<void(const Okular::Page *, const QPoint)>(78, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 79, 29 }, { 0x80000000 | 80, 81 },
        }}),
        // Slot 'slotShowTOCMenu'
        QtMocHelpers::SlotData<void(const Okular::DocumentViewport &, const QPoint, const QString &)>(82, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 83, 84 }, { 0x80000000 | 80, 81 }, { QMetaType::QString, 85 },
        }}),
        // Slot 'slotShowProperties'
        QtMocHelpers::SlotData<void()>(86, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowEmbeddedFiles'
        QtMocHelpers::SlotData<void()>(87, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowLeftPanel'
        QtMocHelpers::SlotData<void()>(88, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowBottomBar'
        QtMocHelpers::SlotData<void()>(89, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowPresentation'
        QtMocHelpers::SlotData<void()>(90, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHidePresentation'
        QtMocHelpers::SlotData<void()>(91, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotUpdateHamburgerMenu'
        QtMocHelpers::SlotData<void()>(92, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotExportAs'
        QtMocHelpers::SlotData<void(QAction *)>(93, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 94, 4 },
        }}),
        // Slot 'slotImportPSFile'
        QtMocHelpers::SlotData<bool()>(95, 4, QMC::AccessProtected, QMetaType::Bool),
        // Slot 'slotAboutBackend'
        QtMocHelpers::SlotData<void()>(96, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotReload'
        QtMocHelpers::SlotData<void()>(97, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(98, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'cannotQuit'
        QtMocHelpers::SlotData<void()>(99, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShowFindBar'
        QtMocHelpers::SlotData<void()>(100, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHideFindBar'
        QtMocHelpers::SlotData<void()>(101, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotJobStarted'
        QtMocHelpers::SlotData<void(KIO::Job *)>(102, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 103, 104 },
        }}),
        // Slot 'slotJobFinished'
        QtMocHelpers::SlotData<void(KJob *)>(105, 4, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 106, 104 },
        }}),
        // Slot 'loadCancelled'
        QtMocHelpers::SlotData<void(const QString &)>(107, 4, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 108 },
        }}),
        // Slot 'setWindowTitleFromDocument'
        QtMocHelpers::SlotData<void()>(109, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateViewActions'
        QtMocHelpers::SlotData<void()>(110, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateBookmarksActions'
        QtMocHelpers::SlotData<void()>(111, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'enableTOC'
        QtMocHelpers::SlotData<void(bool)>(112, 4, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'slotRebuildBookmarkMenu'
        QtMocHelpers::SlotData<void()>(113, 4, QMC::AccessProtected, QMetaType::Void),
        // Slot 'enableLayers'
        QtMocHelpers::SlotData<void(bool)>(114, 4, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'enableSidebarSignaturesItem'
        QtMocHelpers::SlotData<void(bool)>(115, 4, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Slot 'saveFile'
        QtMocHelpers::SlotData<bool()>(116, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'slotPrint'
        QtMocHelpers::SlotData<void()>(117, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileDirty'
        QtMocHelpers::SlotData<void(const QString &)>(118, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'slotAttemptReload'
        QtMocHelpers::SlotData<bool(bool, const QUrl &)>(119, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Bool, 120 }, { 0x80000000 | 60, 121 },
        }}),
        // Slot 'slotAttemptReload'
        QtMocHelpers::SlotData<bool(bool)>(119, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::Bool, 120 },
        }}),
        // Slot 'slotAttemptReload'
        QtMocHelpers::SlotData<bool()>(119, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'psTransformEnded'
        QtMocHelpers::SlotData<void(int, QProcess::ExitStatus)>(122, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { 0x80000000 | 123, 4 },
        }}),
        // Slot 'errorMessage'
        QtMocHelpers::SlotData<void(const QString &, int)>(124, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 125 }, { QMetaType::Int, 126 },
        }}),
        // Slot 'errorMessage'
        QtMocHelpers::SlotData<void(const QString &)>(124, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 125 },
        }}),
        // Slot 'warningMessage'
        QtMocHelpers::SlotData<void(const QString &, int)>(127, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 125 }, { QMetaType::Int, 126 },
        }}),
        // Slot 'warningMessage'
        QtMocHelpers::SlotData<void(const QString &)>(127, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 125 },
        }}),
        // Slot 'noticeMessage'
        QtMocHelpers::SlotData<void(const QString &, int)>(128, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 125 }, { QMetaType::Int, 126 },
        }}),
        // Slot 'noticeMessage'
        QtMocHelpers::SlotData<void(const QString &)>(128, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 125 },
        }}),
        // Slot 'moveSplitter'
        QtMocHelpers::SlotData<void(const int)>(129, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 130 },
        }}),
        // Slot 'finishSigning'
        QtMocHelpers::SlotData<void()>(131, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAccessibilityPreferences'
        QtMocHelpers::SlotData<void()>(132, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAnnotationPreferences'
        QtMocHelpers::SlotData<void()>(133, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHandleActivatedSourceReference'
        QtMocHelpers::SlotData<void(const QString &, int, int, bool *)>(134, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 8 }, { QMetaType::Int, 135 }, { 0x80000000 | 136, 137 },
        }}),
        // Method 'activateTabIfAlreadyOpenFile'
        QtMocHelpers::MethodData<bool() const>(138, 4, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Part, qt_meta_tag_ZN6Okular4PartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Okular::Part::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::ReadWritePart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular4PartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular4PartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Okular4PartE_t>.metaTypes,
    nullptr
} };

void Okular::Part::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Part *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enablePrintAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->openSourceReference((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->viewerMenuStateChange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->enableCloseAction((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->mimeTypeChanged((*reinterpret_cast<std::add_pointer_t<QMimeType>>(_a[1]))); break;
        case 5: _t->urlsDropped((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 6: _t->fitWindowToPage((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2]))); break;
        case 7: _t->maxRecentItemsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->requestOpenNewlySignedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->goToPage((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 10: _t->openDocument((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: { uint _r = _t->pages();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 12: { uint _r = _t->currentPage();
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->currentDocument();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 14: { QString _r = _t->documentMetaData((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->slotPreferences(); break;
        case 16: _t->slotFind(); break;
        case 17: _t->slotPrintPreview(); break;
        case 18: _t->slotPreviousPage(); break;
        case 19: _t->slotNextPage(); break;
        case 20: _t->slotGotoFirst(); break;
        case 21: _t->slotGotoLast(); break;
        case 22: _t->slotTogglePresentation(); break;
        case 23: _t->slotToggleChangeColors(); break;
        case 24: _t->slotSetChangeColors((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->reload(); break;
        case 26: _t->enableStartWithPrint(); break;
        case 27: _t->enableExitAfterPrint(); break;
        case 28: _t->enableStartWithFind((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 29: _t->setEditorCmd((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 30: _t->slotOpenContainingFolder(); break;
        case 31: _t->slotCopyAnnotation(); break;
        case 32: _t->slotCopyTextSelectionOrAnnotation(); break;
        case 33: _t->slotPasteAnnotation(); break;
        case 34: _t->openUrlFromDocument((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 35: _t->openUrlFromBookmarks((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 36: _t->handleDroppedUrls((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 37: _t->slotGoToPage(); break;
        case 38: _t->slotHistoryBack(); break;
        case 39: _t->slotHistoryNext(); break;
        case 40: _t->slotAddBookmark(); break;
        case 41: _t->slotRenameBookmarkFromMenu(); break;
        case 42: _t->slotRemoveBookmarkFromMenu(); break;
        case 43: _t->slotRenameCurrentViewportBookmark(); break;
        case 44: _t->slotPreviousBookmark(); break;
        case 45: _t->slotNextBookmark(); break;
        case 46: _t->slotFindNext(); break;
        case 47: _t->slotFindPrev(); break;
        case 48: { bool _r = _t->slotSaveFileAs((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 49: { bool _r = _t->slotSaveFileAs();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->slotNewConfig(); break;
        case 51: _t->slotShowMenu((*reinterpret_cast<std::add_pointer_t<const Okular::Page*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 52: _t->slotShowTOCMenu((*reinterpret_cast<std::add_pointer_t<Okular::DocumentViewport>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 53: _t->slotShowProperties(); break;
        case 54: _t->slotShowEmbeddedFiles(); break;
        case 55: _t->slotShowLeftPanel(); break;
        case 56: _t->slotShowBottomBar(); break;
        case 57: _t->slotShowPresentation(); break;
        case 58: _t->slotHidePresentation(); break;
        case 59: _t->slotUpdateHamburgerMenu(); break;
        case 60: _t->slotExportAs((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 61: { bool _r = _t->slotImportPSFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 62: _t->slotAboutBackend(); break;
        case 63: _t->slotReload(); break;
        case 64: _t->close(); break;
        case 65: _t->cannotQuit(); break;
        case 66: _t->slotShowFindBar(); break;
        case 67: _t->slotHideFindBar(); break;
        case 68: _t->slotJobStarted((*reinterpret_cast<std::add_pointer_t<KIO::Job*>>(_a[1]))); break;
        case 69: _t->slotJobFinished((*reinterpret_cast<std::add_pointer_t<KJob*>>(_a[1]))); break;
        case 70: _t->loadCancelled((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 71: _t->setWindowTitleFromDocument(); break;
        case 72: _t->updateViewActions(); break;
        case 73: _t->updateBookmarksActions(); break;
        case 74: _t->enableTOC((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 75: _t->slotRebuildBookmarkMenu(); break;
        case 76: _t->enableLayers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 77: _t->enableSidebarSignaturesItem((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 78: { bool _r = _t->saveFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 79: _t->slotPrint(); break;
        case 80: _t->slotFileDirty((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 81: { bool _r = _t->slotAttemptReload((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 82: { bool _r = _t->slotAttemptReload((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 83: { bool _r = _t->slotAttemptReload();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 84: _t->psTransformEnded((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QProcess::ExitStatus>>(_a[2]))); break;
        case 85: _t->errorMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 86: _t->errorMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 87: _t->warningMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 88: _t->warningMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 89: _t->noticeMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 90: _t->noticeMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 91: _t->moveSplitter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 92: _t->finishSigning(); break;
        case 93: _t->slotAccessibilityPreferences(); break;
        case 94: _t->slotAnnotationPreferences(); break;
        case 95: _t->slotHandleActivatedSourceReference((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[4]))); break;
        case 96: { bool _r = _t->activateTabIfAlreadyOpenFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 60:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 68:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIO::Job* >(); break;
            }
            break;
        case 69:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KJob* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Part::*)(bool )>(_a, &Part::enablePrintAction, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const QString & , int , int )>(_a, &Part::openSourceReference, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(bool )>(_a, &Part::viewerMenuStateChange, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(bool )>(_a, &Part::enableCloseAction, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const QMimeType & )>(_a, &Part::mimeTypeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const QList<QUrl> & )>(_a, &Part::urlsDropped, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const QSize , const QSize )>(_a, &Part::fitWindowToPage, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const int )>(_a, &Part::maxRecentItemsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Part::*)(const QString & , int )>(_a, &Part::requestOpenNewlySignedFile, 8))
            return;
    }
}

const QMetaObject *Okular::Part::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Okular::Part::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular4PartE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Okular::DocumentObserver"))
        return static_cast< Okular::DocumentObserver*>(this);
    if (!strcmp(_clname, "KDocumentViewer"))
        return static_cast< KDocumentViewer*>(this);
    if (!strcmp(_clname, "Okular::ViewerInterface"))
        return static_cast< Okular::ViewerInterface*>(this);
    if (!strcmp(_clname, "org.kde.kdocumentviewer/0.1"))
        return static_cast< KDocumentViewer*>(this);
    if (!strcmp(_clname, "org.kde.okular.ViewerInterface/0.1"))
        return static_cast< Okular::ViewerInterface*>(this);
    return KParts::ReadWritePart::qt_metacast(_clname);
}

int Okular::Part::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ReadWritePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 97)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 97;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 97)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 97;
    }
    return _id;
}

// SIGNAL 0
void Okular::Part::enablePrintAction(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Okular::Part::openSourceReference(const QString & _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void Okular::Part::viewerMenuStateChange(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Okular::Part::enableCloseAction(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Okular::Part::mimeTypeChanged(const QMimeType & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Okular::Part::urlsDropped(const QList<QUrl> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Okular::Part::fitWindowToPage(const QSize _t1, const QSize _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void Okular::Part::maxRecentItemsChanged(const int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Okular::Part::requestOpenNewlySignedFile(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}
QT_WARNING_POP
