/********************************************************************************
** Form generated from reading UI file 'selectcertificatedialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SELECTCERTIFICATEDIALOG_H
#define UI_SELECTCERTIFICATEDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SelectCertificateDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *toggleTypes;
    QRadioButton *allCertificateOption;
    QRadioButton *qesCertificates;
    QRadioButton *pgpOption;
    QListView *list;
    QFormLayout *formLayout;
    QLabel *reasonLabel;
    QLineEdit *reasonInput;
    QLabel *locationLabel;
    QLineEdit *locationInput;
    QLabel *backgroundLabel;
    QHBoxLayout *horizontalLayout;
    QLineEdit *backgroundInput;
    QPushButton *backgroundButton;
    QLabel *recentLabel;
    QListView *recentBackgrounds;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SelectCertificateDialog)
    {
        if (SelectCertificateDialog->objectName().isEmpty())
            SelectCertificateDialog->setObjectName("SelectCertificateDialog");
        SelectCertificateDialog->resize(400, 659);
        verticalLayout = new QVBoxLayout(SelectCertificateDialog);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(SelectCertificateDialog);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        toggleTypes = new QHBoxLayout();
        toggleTypes->setObjectName("toggleTypes");
        allCertificateOption = new QRadioButton(SelectCertificateDialog);
        allCertificateOption->setObjectName("allCertificateOption");
        allCertificateOption->setChecked(true);

        toggleTypes->addWidget(allCertificateOption);

        qesCertificates = new QRadioButton(SelectCertificateDialog);
        qesCertificates->setObjectName("qesCertificates");

        toggleTypes->addWidget(qesCertificates);

        pgpOption = new QRadioButton(SelectCertificateDialog);
        pgpOption->setObjectName("pgpOption");

        toggleTypes->addWidget(pgpOption);


        verticalLayout->addLayout(toggleTypes);

        list = new QListView(SelectCertificateDialog);
        list->setObjectName("list");
        list->setAlternatingRowColors(true);
        list->setSelectionBehavior(QAbstractItemView::SelectRows);
        list->setLayoutMode(QListView::SinglePass);

        verticalLayout->addWidget(list);

        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        reasonLabel = new QLabel(SelectCertificateDialog);
        reasonLabel->setObjectName("reasonLabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, reasonLabel);

        reasonInput = new QLineEdit(SelectCertificateDialog);
        reasonInput->setObjectName("reasonInput");
        reasonInput->setClearButtonEnabled(true);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, reasonInput);

        locationLabel = new QLabel(SelectCertificateDialog);
        locationLabel->setObjectName("locationLabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, locationLabel);

        locationInput = new QLineEdit(SelectCertificateDialog);
        locationInput->setObjectName("locationInput");
        locationInput->setClearButtonEnabled(true);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, locationInput);

        backgroundLabel = new QLabel(SelectCertificateDialog);
        backgroundLabel->setObjectName("backgroundLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, backgroundLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        backgroundInput = new QLineEdit(SelectCertificateDialog);
        backgroundInput->setObjectName("backgroundInput");
        backgroundInput->setClearButtonEnabled(true);

        horizontalLayout->addWidget(backgroundInput);

        backgroundButton = new QPushButton(SelectCertificateDialog);
        backgroundButton->setObjectName("backgroundButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("folder-image")));
        backgroundButton->setIcon(icon);

        horizontalLayout->addWidget(backgroundButton);


        formLayout->setLayout(2, QFormLayout::ItemRole::FieldRole, horizontalLayout);

        recentLabel = new QLabel(SelectCertificateDialog);
        recentLabel->setObjectName("recentLabel");
        recentLabel->setTextFormat(Qt::AutoText);
        recentLabel->setWordWrap(false);

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, recentLabel);

        recentBackgrounds = new QListView(SelectCertificateDialog);
        recentBackgrounds->setObjectName("recentBackgrounds");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, recentBackgrounds);


        verticalLayout->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(SelectCertificateDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(SelectCertificateDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, SelectCertificateDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SelectCertificateDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SelectCertificateDialog);
    } // setupUi

    void retranslateUi(QDialog *SelectCertificateDialog)
    {
        SelectCertificateDialog->setWindowTitle(tr2i18n("Certificates", nullptr));
        label->setText(tr2i18n("Select certificate to sign with:", nullptr));
        allCertificateOption->setText(tr2i18n("All", nullptr));
        qesCertificates->setText(tr2i18n("Only QES certificates", nullptr));
        pgpOption->setText(tr2i18n("Only PGP keys", "@option:radio"));
        reasonLabel->setText(tr2i18n("Reason:", nullptr));
        reasonInput->setPlaceholderText(tr2i18n("optional", nullptr));
        locationLabel->setText(tr2i18n("Location:", nullptr));
        locationInput->setPlaceholderText(tr2i18n("optional", nullptr));
        backgroundLabel->setText(tr2i18n("Background:", nullptr));
        backgroundInput->setPlaceholderText(tr2i18n("optional", nullptr));
        backgroundButton->setText(tr2i18n("Choose...", nullptr));
        recentLabel->setText(tr2i18n("Recent backgrounds:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SelectCertificateDialog: public Ui_SelectCertificateDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SELECTCERTIFICATEDIALOG_H

