/****************************************************************************
** Meta object code from reading C++ file 'document.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/core/document.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'document.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Okular8DocumentE_t {};
} // unnamed namespace

template <> constexpr inline auto Okular::Document::qt_create_metaobjectdata<qt_meta_tag_ZN6Okular8DocumentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Okular::Document",
        "aboutToClose",
        "",
        "close",
        "requestPrint",
        "requestSaveAs",
        "quit",
        "linkFind",
        "linkGoToPage",
        "linkPresentation",
        "linkEndPresentation",
        "openUrl",
        "QUrl",
        "url",
        "error",
        "text",
        "duration",
        "warning",
        "notice",
        "gotFont",
        "Okular::FontInfo",
        "font",
        "fontReadingProgress",
        "page",
        "fontReadingEnded",
        "searchFinished",
        "searchID",
        "Okular::Document::SearchStatus",
        "endStatus",
        "sourceReferenceActivated",
        "absFileName",
        "line",
        "col",
        "bool*",
        "handled",
        "processMovieAction",
        "const Okular::MovieAction*",
        "action",
        "canUndoChanged",
        "undoAvailable",
        "canRedoChanged",
        "redoAvailable",
        "undoHistoryCleanChanged",
        "clean",
        "processRenditionAction",
        "const Okular::RenditionAction*",
        "annotationContentsChangedByUndoRedo",
        "Okular::Annotation*",
        "annotation",
        "contents",
        "cursorPos",
        "anchorPos",
        "formTextChangedByUndoRedo",
        "Okular::FormFieldText*",
        "form",
        "formListChangedByUndoRedo",
        "Okular::FormFieldChoice*",
        "QList<int>",
        "choices",
        "formComboChangedByUndoRedo",
        "formButtonsChangedByUndoRedo",
        "QList<Okular::FormFieldButton*>",
        "formButtons",
        "refreshFormWidget",
        "Okular::FormField*",
        "field",
        "setRotation",
        "rotation",
        "setPageSize",
        "Okular::PageSize",
        "size",
        "cancelSearch",
        "undo",
        "redo",
        "editFormText",
        "pageNumber",
        "newContents",
        "newCursorPos",
        "prevCursorPos",
        "prevAnchorPos",
        "oldContents",
        "editFormList",
        "newChoices",
        "editFormCombo",
        "newText",
        "editFormButtons",
        "QList<bool>",
        "newButtonStates",
        "reloadDocument",
        "requestSignedRevisionData",
        "Okular::SignatureInfo",
        "info",
        "refreshPixmaps"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'aboutToClose'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'close'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestPrint'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestSaveAs'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'quit'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'linkFind'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'linkGoToPage'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'linkPresentation'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'linkEndPresentation'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openUrl'
        QtMocHelpers::SignalData<void(const QUrl &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QString &, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'warning'
        QtMocHelpers::SignalData<void(const QString &, int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'notice'
        QtMocHelpers::SignalData<void(const QString &, int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'gotFont'
        QtMocHelpers::SignalData<void(const Okular::FontInfo &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Signal 'fontReadingProgress'
        QtMocHelpers::SignalData<void(int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 },
        }}),
        // Signal 'fontReadingEnded'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'searchFinished'
        QtMocHelpers::SignalData<void(int, Okular::Document::SearchStatus)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 26 }, { 0x80000000 | 27, 28 },
        }}),
        // Signal 'sourceReferenceActivated'
        QtMocHelpers::SignalData<void(const QString &, int, int, bool *)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { QMetaType::Int, 31 }, { QMetaType::Int, 32 }, { 0x80000000 | 33, 34 },
        }}),
        // Signal 'processMovieAction'
        QtMocHelpers::SignalData<void(const Okular::MovieAction *)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 36, 37 },
        }}),
        // Signal 'canUndoChanged'
        QtMocHelpers::SignalData<void(bool)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 39 },
        }}),
        // Signal 'canRedoChanged'
        QtMocHelpers::SignalData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 41 },
        }}),
        // Signal 'undoHistoryCleanChanged'
        QtMocHelpers::SignalData<void(bool)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Signal 'processRenditionAction'
        QtMocHelpers::SignalData<void(const Okular::RenditionAction *)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 45, 37 },
        }}),
        // Signal 'annotationContentsChangedByUndoRedo'
        QtMocHelpers::SignalData<void(Okular::Annotation *, const QString &, int, int)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 47, 48 }, { QMetaType::QString, 49 }, { QMetaType::Int, 50 }, { QMetaType::Int, 51 },
        }}),
        // Signal 'formTextChangedByUndoRedo'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldText *, const QString &, int, int)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 }, { 0x80000000 | 53, 54 }, { QMetaType::QString, 49 }, { QMetaType::Int, 50 },
            { QMetaType::Int, 51 },
        }}),
        // Signal 'formListChangedByUndoRedo'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldChoice *, const QList<int> &)>(55, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 }, { 0x80000000 | 56, 54 }, { 0x80000000 | 57, 58 },
        }}),
        // Signal 'formComboChangedByUndoRedo'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldChoice *, const QString &, int, int)>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 }, { 0x80000000 | 56, 54 }, { QMetaType::QString, 15 }, { QMetaType::Int, 50 },
            { QMetaType::Int, 51 },
        }}),
        // Signal 'formButtonsChangedByUndoRedo'
        QtMocHelpers::SignalData<void(int, const QList<Okular::FormFieldButton*> &)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 23 }, { 0x80000000 | 61, 62 },
        }}),
        // Signal 'refreshFormWidget'
        QtMocHelpers::SignalData<void(Okular::FormField *)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 65 },
        }}),
        // Slot 'setRotation'
        QtMocHelpers::SlotData<void(int)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 67 },
        }}),
        // Slot 'setPageSize'
        QtMocHelpers::SlotData<void(const Okular::PageSize &)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 69, 70 },
        }}),
        // Slot 'cancelSearch'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'undo'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'redo'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editFormText'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldText *, const QString &, int, int, int)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 75 }, { 0x80000000 | 53, 54 }, { QMetaType::QString, 76 }, { QMetaType::Int, 77 },
            { QMetaType::Int, 78 }, { QMetaType::Int, 79 },
        }}),
        // Slot 'editFormText'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldText *, const QString &, int, int, int, const QString &)>(74, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 75 }, { 0x80000000 | 53, 54 }, { QMetaType::QString, 76 }, { QMetaType::Int, 77 },
            { QMetaType::Int, 78 }, { QMetaType::Int, 79 }, { QMetaType::QString, 80 },
        }}),
        // Slot 'editFormList'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldChoice *, const QList<int> &)>(81, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 75 }, { 0x80000000 | 56, 54 }, { 0x80000000 | 57, 82 },
        }}),
        // Slot 'editFormCombo'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldChoice *, const QString &, int, int, int)>(83, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 75 }, { 0x80000000 | 56, 54 }, { QMetaType::QString, 84 }, { QMetaType::Int, 77 },
            { QMetaType::Int, 78 }, { QMetaType::Int, 79 },
        }}),
        // Slot 'editFormButtons'
        QtMocHelpers::SlotData<void(int, const QList<Okular::FormFieldButton*> &, const QList<bool> &)>(85, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 75 }, { 0x80000000 | 61, 62 }, { 0x80000000 | 86, 87 },
        }}),
        // Slot 'reloadDocument'
        QtMocHelpers::SlotData<void() const>(88, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestSignedRevisionData'
        QtMocHelpers::SlotData<QByteArray(const Okular::SignatureInfo &)>(89, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { 0x80000000 | 90, 91 },
        }}),
        // Slot 'refreshPixmaps'
        QtMocHelpers::SlotData<void(int)>(92, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 75 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Document, qt_meta_tag_ZN6Okular8DocumentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Okular::Document::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular8DocumentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular8DocumentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Okular8DocumentE_t>.metaTypes,
    nullptr
} };

void Okular::Document::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Document *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->aboutToClose(); break;
        case 1: _t->close(); break;
        case 2: _t->requestPrint(); break;
        case 3: _t->requestSaveAs(); break;
        case 4: _t->quit(); break;
        case 5: _t->linkFind(); break;
        case 6: _t->linkGoToPage(); break;
        case 7: _t->linkPresentation(); break;
        case 8: _t->linkEndPresentation(); break;
        case 9: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 10: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 11: _t->warning((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 12: _t->notice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->gotFont((*reinterpret_cast<std::add_pointer_t<Okular::FontInfo>>(_a[1]))); break;
        case 14: _t->fontReadingProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->fontReadingEnded(); break;
        case 16: _t->searchFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::Document::SearchStatus>>(_a[2]))); break;
        case 17: _t->sourceReferenceActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[4]))); break;
        case 18: _t->processMovieAction((*reinterpret_cast<std::add_pointer_t<const Okular::MovieAction*>>(_a[1]))); break;
        case 19: _t->canUndoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->canRedoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->undoHistoryCleanChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->processRenditionAction((*reinterpret_cast<std::add_pointer_t<const Okular::RenditionAction*>>(_a[1]))); break;
        case 23: _t->annotationContentsChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<Okular::Annotation*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 24: _t->formTextChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 25: _t->formListChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 26: _t->formComboChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 27: _t->formButtonsChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Okular::FormFieldButton*>>>(_a[2]))); break;
        case 28: _t->refreshFormWidget((*reinterpret_cast<std::add_pointer_t<Okular::FormField*>>(_a[1]))); break;
        case 29: _t->setRotation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 30: _t->setPageSize((*reinterpret_cast<std::add_pointer_t<Okular::PageSize>>(_a[1]))); break;
        case 31: _t->cancelSearch(); break;
        case 32: _t->undo(); break;
        case 33: _t->redo(); break;
        case 34: _t->editFormText((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6]))); break;
        case 35: _t->editFormText((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7]))); break;
        case 36: _t->editFormList((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 37: _t->editFormCombo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6]))); break;
        case 38: _t->editFormButtons((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Okular::FormFieldButton*>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<bool>>>(_a[3]))); break;
        case 39: _t->reloadDocument(); break;
        case 40: { QByteArray _r = _t->requestSignedRevisionData((*reinterpret_cast<std::add_pointer_t<Okular::SignatureInfo>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 41: _t->refreshPixmaps((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<bool> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::aboutToClose, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::close, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::requestPrint, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::requestSaveAs, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::quit, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::linkFind, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::linkGoToPage, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::linkPresentation, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::linkEndPresentation, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const QUrl & )>(_a, &Document::openUrl, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const QString & , int )>(_a, &Document::error, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const QString & , int )>(_a, &Document::warning, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const QString & , int )>(_a, &Document::notice, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const Okular::FontInfo & )>(_a, &Document::gotFont, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(int )>(_a, &Document::fontReadingProgress, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)()>(_a, &Document::fontReadingEnded, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(int , Okular::Document::SearchStatus )>(_a, &Document::searchFinished, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const QString & , int , int , bool * )>(_a, &Document::sourceReferenceActivated, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const Okular::MovieAction * )>(_a, &Document::processMovieAction, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(bool )>(_a, &Document::canUndoChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(bool )>(_a, &Document::canRedoChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(bool )>(_a, &Document::undoHistoryCleanChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(const Okular::RenditionAction * )>(_a, &Document::processRenditionAction, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(Okular::Annotation * , const QString & , int , int )>(_a, &Document::annotationContentsChangedByUndoRedo, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(int , Okular::FormFieldText * , const QString & , int , int )>(_a, &Document::formTextChangedByUndoRedo, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(int , Okular::FormFieldChoice * , const QList<int> & )>(_a, &Document::formListChangedByUndoRedo, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(int , Okular::FormFieldChoice * , const QString & , int , int )>(_a, &Document::formComboChangedByUndoRedo, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(int , const QList<Okular::FormFieldButton*> & )>(_a, &Document::formButtonsChangedByUndoRedo, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (Document::*)(Okular::FormField * )>(_a, &Document::refreshFormWidget, 28))
            return;
    }
}

const QMetaObject *Okular::Document::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Okular::Document::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular8DocumentE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Okular::Document::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    }
    return _id;
}

// SIGNAL 0
void Okular::Document::aboutToClose()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Okular::Document::close()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Okular::Document::requestPrint()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Okular::Document::requestSaveAs()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Okular::Document::quit()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Okular::Document::linkFind()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Okular::Document::linkGoToPage()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Okular::Document::linkPresentation()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Okular::Document::linkEndPresentation()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Okular::Document::openUrl(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Okular::Document::error(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void Okular::Document::warning(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void Okular::Document::notice(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void Okular::Document::gotFont(const Okular::FontInfo & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void Okular::Document::fontReadingProgress(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void Okular::Document::fontReadingEnded()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void Okular::Document::searchFinished(int _t1, Okular::Document::SearchStatus _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2);
}

// SIGNAL 17
void Okular::Document::sourceReferenceActivated(const QString & _t1, int _t2, int _t3, bool * _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 18
void Okular::Document::processMovieAction(const Okular::MovieAction * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void Okular::Document::canUndoChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void Okular::Document::canRedoChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void Okular::Document::undoHistoryCleanChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void Okular::Document::processRenditionAction(const Okular::RenditionAction * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void Okular::Document::annotationContentsChangedByUndoRedo(Okular::Annotation * _t1, const QString & _t2, int _t3, int _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 24
void Okular::Document::formTextChangedByUndoRedo(int _t1, Okular::FormFieldText * _t2, const QString & _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 25
void Okular::Document::formListChangedByUndoRedo(int _t1, Okular::FormFieldChoice * _t2, const QList<int> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 25, nullptr, _t1, _t2, _t3);
}

// SIGNAL 26
void Okular::Document::formComboChangedByUndoRedo(int _t1, Okular::FormFieldChoice * _t2, const QString & _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 26, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 27
void Okular::Document::formButtonsChangedByUndoRedo(int _t1, const QList<Okular::FormFieldButton*> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 27, nullptr, _t1, _t2);
}

// SIGNAL 28
void Okular::Document::refreshFormWidget(Okular::FormField * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}
QT_WARNING_POP
