/*
    SPDX-FileCopyrightText: 2014 Laurent Montel <montel@kde.org>
    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "fax_debug.h"
Q_LOGGING_CATEGORY(FAX_LOG, "org.kde.okular.generators.fax", QtWarningMsg)
