/****************************************************************************
** Meta object code from reading C++ file 'presentationwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/part/presentationwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'presentationwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18PresentationWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto PresentationWidget::qt_create_metaobjectdata<qt_meta_tag_ZN18PresentationWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PresentationWidget",
        "slotFind",
        "",
        "slotNextPage",
        "slotPrevPage",
        "slotFirstPage",
        "slotLastPage",
        "slotHideOverlay",
        "slotTransitionStep",
        "slotDelayedEvents",
        "slotPageChanged",
        "clearDrawings",
        "chooseScreen",
        "QAction*",
        "toggleBlackScreenMode",
        "slotProcessMovieAction",
        "const Okular::MovieAction*",
        "action",
        "slotProcessRenditionAction",
        "const Okular::RenditionAction*",
        "slotTogglePlayPause",
        "slotChangeDrawingToolEngine",
        "QDomElement",
        "element",
        "slotAddDrawingToolActions"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotFind'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNextPage'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPrevPage'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFirstPage'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotLastPage'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHideOverlay'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTransitionStep'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDelayedEvents'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPageChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'clearDrawings'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'chooseScreen'
        QtMocHelpers::SlotData<void(QAction *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Slot 'toggleBlackScreenMode'
        QtMocHelpers::SlotData<void(bool)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotProcessMovieAction'
        QtMocHelpers::SlotData<void(const Okular::MovieAction *)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Slot 'slotProcessRenditionAction'
        QtMocHelpers::SlotData<void(const Okular::RenditionAction *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 17 },
        }}),
        // Slot 'slotTogglePlayPause'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotChangeDrawingToolEngine'
        QtMocHelpers::SlotData<void(const QDomElement &)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'slotAddDrawingToolActions'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PresentationWidget, qt_meta_tag_ZN18PresentationWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PresentationWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PresentationWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PresentationWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18PresentationWidgetE_t>.metaTypes,
    nullptr
} };

void PresentationWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PresentationWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotFind(); break;
        case 1: _t->slotNextPage(); break;
        case 2: _t->slotPrevPage(); break;
        case 3: _t->slotFirstPage(); break;
        case 4: _t->slotLastPage(); break;
        case 5: _t->slotHideOverlay(); break;
        case 6: _t->slotTransitionStep(); break;
        case 7: _t->slotDelayedEvents(); break;
        case 8: _t->slotPageChanged(); break;
        case 9: _t->clearDrawings(); break;
        case 10: _t->chooseScreen((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 11: _t->toggleBlackScreenMode((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->slotProcessMovieAction((*reinterpret_cast<std::add_pointer_t<const Okular::MovieAction*>>(_a[1]))); break;
        case 13: _t->slotProcessRenditionAction((*reinterpret_cast<std::add_pointer_t<const Okular::RenditionAction*>>(_a[1]))); break;
        case 14: _t->slotTogglePlayPause(); break;
        case 15: _t->slotChangeDrawingToolEngine((*reinterpret_cast<std::add_pointer_t<QDomElement>>(_a[1]))); break;
        case 16: _t->slotAddDrawingToolActions(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *PresentationWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PresentationWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18PresentationWidgetE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Okular::DocumentObserver"))
        return static_cast< Okular::DocumentObserver*>(this);
    return QWidget::qt_metacast(_clname);
}

int PresentationWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
