/****************************************************************************
** Meta object code from reading C++ file 'formwidgets.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/part/formwidgets.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'formwidgets.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21FormWidgetsControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto FormWidgetsController::qt_create_metaobjectdata<qt_meta_tag_ZN21FormWidgetsControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FormWidgetsController",
        "changed",
        "",
        "pageNumber",
        "requestUndo",
        "requestRedo",
        "canUndoChanged",
        "undoAvailable",
        "canRedoChanged",
        "redoAvailable",
        "formTextChangedByWidget",
        "Okular::FormFieldText*",
        "form",
        "newContents",
        "newCursorPos",
        "prevCursorPos",
        "prevAnchorPos",
        "oldContents",
        "formTextChangedByUndoRedo",
        "contents",
        "cursorPos",
        "anchorPos",
        "formListChangedByWidget",
        "Okular::FormFieldChoice*",
        "QList<int>",
        "newChoices",
        "formListChangedByUndoRedo",
        "choices",
        "formComboChangedByWidget",
        "newText",
        "formComboChangedByUndoRedo",
        "text",
        "formButtonsChangedByWidget",
        "QList<Okular::FormFieldButton*>",
        "formButtons",
        "QList<bool>",
        "newButtonStates",
        "action",
        "Okular::Action*",
        "mouseAction",
        "Okular::FormField*",
        "Okular::Document::MouseEventType",
        "fieldMouseEventType",
        "refreshFormWidget",
        "slotButtonClicked",
        "QAbstractButton*",
        "button",
        "slotFormButtonsChangedByUndoRedo"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'changed'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'requestUndo'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestRedo'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canUndoChanged'
        QtMocHelpers::SignalData<void(bool)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Signal 'canRedoChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'formTextChangedByWidget'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldText *, const QString &, int, int, int, const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 11, 12 }, { QMetaType::QString, 13 }, { QMetaType::Int, 14 },
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 }, { QMetaType::QString, 17 },
        }}),
        // Signal 'formTextChangedByUndoRedo'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldText *, const QString &, int, int)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 11, 12 }, { QMetaType::QString, 19 }, { QMetaType::Int, 20 },
            { QMetaType::Int, 21 },
        }}),
        // Signal 'formListChangedByWidget'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldChoice *, const QList<int> &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 23, 12 }, { 0x80000000 | 24, 25 },
        }}),
        // Signal 'formListChangedByUndoRedo'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldChoice *, const QList<int> &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 23, 12 }, { 0x80000000 | 24, 27 },
        }}),
        // Signal 'formComboChangedByWidget'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldChoice *, const QString &, int, int, int)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 23, 12 }, { QMetaType::QString, 29 }, { QMetaType::Int, 14 },
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'formComboChangedByUndoRedo'
        QtMocHelpers::SignalData<void(int, Okular::FormFieldChoice *, const QString &, int, int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 23, 12 }, { QMetaType::QString, 31 }, { QMetaType::Int, 20 },
            { QMetaType::Int, 21 },
        }}),
        // Signal 'formButtonsChangedByWidget'
        QtMocHelpers::SignalData<void(int, const QList<Okular::FormFieldButton*> &, const QList<bool> &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 33, 34 }, { 0x80000000 | 35, 36 },
        }}),
        // Signal 'action'
        QtMocHelpers::SignalData<void(Okular::Action *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 37 },
        }}),
        // Signal 'mouseAction'
        QtMocHelpers::SignalData<void(Okular::Action *, Okular::FormField *, Okular::Document::MouseEventType)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 37 }, { 0x80000000 | 40, 12 }, { 0x80000000 | 41, 42 },
        }}),
        // Signal 'refreshFormWidget'
        QtMocHelpers::SignalData<void(Okular::FormField *)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 12 },
        }}),
        // Slot 'slotButtonClicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'slotFormButtonsChangedByUndoRedo'
        QtMocHelpers::SlotData<void(int, const QList<Okular::FormFieldButton*> &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 33, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FormWidgetsController, qt_meta_tag_ZN21FormWidgetsControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FormWidgetsController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21FormWidgetsControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21FormWidgetsControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21FormWidgetsControllerE_t>.metaTypes,
    nullptr
} };

void FormWidgetsController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FormWidgetsController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->changed((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->requestUndo(); break;
        case 2: _t->requestRedo(); break;
        case 3: _t->canUndoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->canRedoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->formTextChangedByWidget((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[7]))); break;
        case 6: _t->formTextChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 7: _t->formListChangedByWidget((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 8: _t->formListChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 9: _t->formComboChangedByWidget((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[6]))); break;
        case 10: _t->formComboChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 11: _t->formButtonsChangedByWidget((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Okular::FormFieldButton*>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<bool>>>(_a[3]))); break;
        case 12: _t->action((*reinterpret_cast<std::add_pointer_t<Okular::Action*>>(_a[1]))); break;
        case 13: _t->mouseAction((*reinterpret_cast<std::add_pointer_t<Okular::Action*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormField*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Okular::Document::MouseEventType>>(_a[3]))); break;
        case 14: _t->refreshFormWidget((*reinterpret_cast<std::add_pointer_t<Okular::FormField*>>(_a[1]))); break;
        case 15: _t->slotButtonClicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 16: _t->slotFormButtonsChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Okular::FormFieldButton*>>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<bool> >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractButton* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int )>(_a, &FormWidgetsController::changed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)()>(_a, &FormWidgetsController::requestUndo, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)()>(_a, &FormWidgetsController::requestRedo, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(bool )>(_a, &FormWidgetsController::canUndoChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(bool )>(_a, &FormWidgetsController::canRedoChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int , Okular::FormFieldText * , const QString & , int , int , int , const QString & )>(_a, &FormWidgetsController::formTextChangedByWidget, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int , Okular::FormFieldText * , const QString & , int , int )>(_a, &FormWidgetsController::formTextChangedByUndoRedo, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int , Okular::FormFieldChoice * , const QList<int> & )>(_a, &FormWidgetsController::formListChangedByWidget, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int , Okular::FormFieldChoice * , const QList<int> & )>(_a, &FormWidgetsController::formListChangedByUndoRedo, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int , Okular::FormFieldChoice * , const QString & , int , int , int )>(_a, &FormWidgetsController::formComboChangedByWidget, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int , Okular::FormFieldChoice * , const QString & , int , int )>(_a, &FormWidgetsController::formComboChangedByUndoRedo, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(int , const QList<Okular::FormFieldButton*> & , const QList<bool> & )>(_a, &FormWidgetsController::formButtonsChangedByWidget, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(Okular::Action * )>(_a, &FormWidgetsController::action, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(Okular::Action * , Okular::FormField * , Okular::Document::MouseEventType )>(_a, &FormWidgetsController::mouseAction, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormWidgetsController::*)(Okular::FormField * )>(_a, &FormWidgetsController::refreshFormWidget, 14))
            return;
    }
}

const QMetaObject *FormWidgetsController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FormWidgetsController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21FormWidgetsControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int FormWidgetsController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void FormWidgetsController::changed(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FormWidgetsController::requestUndo()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FormWidgetsController::requestRedo()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void FormWidgetsController::canUndoChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void FormWidgetsController::canRedoChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void FormWidgetsController::formTextChangedByWidget(int _t1, Okular::FormFieldText * _t2, const QString & _t3, int _t4, int _t5, int _t6, const QString & _t7)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7);
}

// SIGNAL 6
void FormWidgetsController::formTextChangedByUndoRedo(int _t1, Okular::FormFieldText * _t2, const QString & _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 7
void FormWidgetsController::formListChangedByWidget(int _t1, Okular::FormFieldChoice * _t2, const QList<int> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void FormWidgetsController::formListChangedByUndoRedo(int _t1, Okular::FormFieldChoice * _t2, const QList<int> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}

// SIGNAL 9
void FormWidgetsController::formComboChangedByWidget(int _t1, Okular::FormFieldChoice * _t2, const QString & _t3, int _t4, int _t5, int _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 10
void FormWidgetsController::formComboChangedByUndoRedo(int _t1, Okular::FormFieldChoice * _t2, const QString & _t3, int _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 11
void FormWidgetsController::formButtonsChangedByWidget(int _t1, const QList<Okular::FormFieldButton*> & _t2, const QList<bool> & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2, _t3);
}

// SIGNAL 12
void FormWidgetsController::action(Okular::Action * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void FormWidgetsController::mouseAction(Okular::Action * _t1, Okular::FormField * _t2, Okular::Document::MouseEventType _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2, _t3);
}

// SIGNAL 14
void FormWidgetsController::refreshFormWidget(Okular::FormField * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN14PushButtonEditE_t {};
} // unnamed namespace

template <> constexpr inline auto PushButtonEdit::qt_create_metaobjectdata<qt_meta_tag_ZN14PushButtonEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PushButtonEdit"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PushButtonEdit, qt_meta_tag_ZN14PushButtonEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PushButtonEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QPushButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PushButtonEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PushButtonEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14PushButtonEditE_t>.metaTypes,
    nullptr
} };

void PushButtonEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PushButtonEdit *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *PushButtonEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PushButtonEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PushButtonEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return QPushButton::qt_metacast(_clname);
}

int PushButtonEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPushButton::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12CheckBoxEditE_t {};
} // unnamed namespace

template <> constexpr inline auto CheckBoxEdit::qt_create_metaobjectdata<qt_meta_tag_ZN12CheckBoxEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CheckBoxEdit"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CheckBoxEdit, qt_meta_tag_ZN12CheckBoxEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CheckBoxEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QCheckBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CheckBoxEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CheckBoxEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12CheckBoxEditE_t>.metaTypes,
    nullptr
} };

void CheckBoxEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CheckBoxEdit *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CheckBoxEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CheckBoxEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12CheckBoxEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return QCheckBox::qt_metacast(_clname);
}

int CheckBoxEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QCheckBox::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15RadioButtonEditE_t {};
} // unnamed namespace

template <> constexpr inline auto RadioButtonEdit::qt_create_metaobjectdata<qt_meta_tag_ZN15RadioButtonEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RadioButtonEdit"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RadioButtonEdit, qt_meta_tag_ZN15RadioButtonEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RadioButtonEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QRadioButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15RadioButtonEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15RadioButtonEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15RadioButtonEditE_t>.metaTypes,
    nullptr
} };

void RadioButtonEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RadioButtonEdit *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *RadioButtonEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RadioButtonEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15RadioButtonEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return QRadioButton::qt_metacast(_clname);
}

int RadioButtonEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRadioButton::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN12FormLineEditE_t {};
} // unnamed namespace

template <> constexpr inline auto FormLineEdit::qt_create_metaobjectdata<qt_meta_tag_ZN12FormLineEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FormLineEdit",
        "slotHandleTextChangedByUndoRedo",
        "",
        "pageNumber",
        "Okular::FormFieldText*",
        "textForm",
        "contents",
        "cursorPos",
        "anchorPos",
        "slotChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotHandleTextChangedByUndoRedo'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldText *, const QString &, int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::Int, 7 },
            { QMetaType::Int, 8 },
        }}),
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FormLineEdit, qt_meta_tag_ZN12FormLineEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FormLineEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QLineEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FormLineEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FormLineEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12FormLineEditE_t>.metaTypes,
    nullptr
} };

void FormLineEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FormLineEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotHandleTextChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 1: _t->slotChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *FormLineEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FormLineEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FormLineEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return QLineEdit::qt_metacast(_clname);
}

int FormLineEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QLineEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN12TextAreaEditE_t {};
} // unnamed namespace

template <> constexpr inline auto TextAreaEdit::qt_create_metaobjectdata<qt_meta_tag_ZN12TextAreaEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TextAreaEdit",
        "slotHandleTextChangedByUndoRedo",
        "",
        "pageNumber",
        "Okular::FormFieldText*",
        "textForm",
        "contents",
        "cursorPos",
        "anchorPos",
        "slotUpdateUndoAndRedoInContextMenu",
        "QMenu*",
        "menu",
        "slotChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotHandleTextChangedByUndoRedo'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldText *, const QString &, int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::QString, 6 }, { QMetaType::Int, 7 },
            { QMetaType::Int, 8 },
        }}),
        // Slot 'slotUpdateUndoAndRedoInContextMenu'
        QtMocHelpers::SlotData<void(QMenu *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextAreaEdit, qt_meta_tag_ZN12TextAreaEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TextAreaEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<KTextEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TextAreaEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TextAreaEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12TextAreaEditE_t>.metaTypes,
    nullptr
} };

void TextAreaEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextAreaEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotHandleTextChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 1: _t->slotUpdateUndoAndRedoInContextMenu((*reinterpret_cast<std::add_pointer_t<QMenu*>>(_a[1]))); break;
        case 2: _t->slotChanged(); break;
        default: ;
        }
    }
}

const QMetaObject *TextAreaEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TextAreaEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TextAreaEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return KTextEdit::qt_metacast(_clname);
}

int TextAreaEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8FileEditE_t {};
} // unnamed namespace

template <> constexpr inline auto FileEdit::qt_create_metaobjectdata<qt_meta_tag_ZN8FileEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileEdit",
        "slotChanged",
        "",
        "slotHandleFileChangedByUndoRedo",
        "pageNumber",
        "Okular::FormFieldText*",
        "form",
        "contents",
        "cursorPos",
        "anchorPos"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotChanged'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHandleFileChangedByUndoRedo'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldText *, const QString &, int, int)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 }, { QMetaType::Int, 8 },
            { QMetaType::Int, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileEdit, qt_meta_tag_ZN8FileEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<KUrlRequester::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8FileEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8FileEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8FileEditE_t>.metaTypes,
    nullptr
} };

void FileEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotChanged(); break;
        case 1: _t->slotHandleFileChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldText*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        default: ;
        }
    }
}

const QMetaObject *FileEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8FileEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return KUrlRequester::qt_metacast(_clname);
}

int FileEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KUrlRequester::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN8ListEditE_t {};
} // unnamed namespace

template <> constexpr inline auto ListEdit::qt_create_metaobjectdata<qt_meta_tag_ZN8ListEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ListEdit",
        "slotSelectionChanged",
        "",
        "slotHandleFormListChangedByUndoRedo",
        "pageNumber",
        "Okular::FormFieldChoice*",
        "listForm",
        "QList<int>",
        "choices"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHandleFormListChangedByUndoRedo'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldChoice *, const QList<int> &)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ListEdit, qt_meta_tag_ZN8ListEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ListEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QListWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ListEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ListEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8ListEditE_t>.metaTypes,
    nullptr
} };

void ListEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ListEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotSelectionChanged(); break;
        case 1: _t->slotHandleFormListChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *ListEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ListEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8ListEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return QListWidget::qt_metacast(_clname);
}

int ListEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN9ComboEditE_t {};
} // unnamed namespace

template <> constexpr inline auto ComboEdit::qt_create_metaobjectdata<qt_meta_tag_ZN9ComboEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ComboEdit",
        "slotValueChanged",
        "",
        "slotHandleFormComboChangedByUndoRedo",
        "pageNumber",
        "Okular::FormFieldChoice*",
        "comboForm",
        "text",
        "cursorPos",
        "anchorPos"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotValueChanged'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHandleFormComboChangedByUndoRedo'
        QtMocHelpers::SlotData<void(int, Okular::FormFieldChoice *, const QString &, int, int)>(3, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 }, { 0x80000000 | 5, 6 }, { QMetaType::QString, 7 }, { QMetaType::Int, 8 },
            { QMetaType::Int, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ComboEdit, qt_meta_tag_ZN9ComboEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ComboEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QComboBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9ComboEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9ComboEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9ComboEditE_t>.metaTypes,
    nullptr
} };

void ComboEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ComboEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotValueChanged(); break;
        case 1: _t->slotHandleFormComboChangedByUndoRedo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormFieldChoice*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        default: ;
        }
    }
}

const QMetaObject *ComboEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ComboEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9ComboEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return QComboBox::qt_metacast(_clname);
}

int ComboEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QComboBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN13SignatureEditE_t {};
} // unnamed namespace

template <> constexpr inline auto SignatureEdit::qt_create_metaobjectdata<qt_meta_tag_ZN13SignatureEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SignatureEdit",
        "slotViewProperties",
        "",
        "signUnsignedSignature"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotViewProperties'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'signUnsignedSignature'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SignatureEdit, qt_meta_tag_ZN13SignatureEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SignatureEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SignatureEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SignatureEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13SignatureEditE_t>.metaTypes,
    nullptr
} };

void SignatureEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SignatureEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotViewProperties(); break;
        case 1: _t->signUnsignedSignature(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *SignatureEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SignatureEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13SignatureEditE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "FormWidgetIface"))
        return static_cast< FormWidgetIface*>(this);
    return QAbstractButton::qt_metacast(_clname);
}

int SignatureEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
