/*  This file is part of the KDE project
    SPDX-FileCopyrightText: 2014 Laurent Montel <montel@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#ifndef SPECTRE_DEBUG_H
#define SPECTRE_DEBUG_H

#include <QLoggingCategory>
Q_DECLARE_LOGGING_CATEGORY(OkularSpectreDebug)

#endif
