/****************************************************************************
** Meta object code from reading C++ file 'pageview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/part/pageview.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8PageViewE_t {};
} // unnamed namespace

template <> constexpr inline auto PageView::qt_create_metaobjectdata<qt_meta_tag_ZN8PageViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PageView",
        "rightClick",
        "",
        "const Okular::Page*",
        "QPoint",
        "mouseBackButtonClick",
        "mouseForwardButtonClick",
        "escPressed",
        "fitWindowToPage",
        "QSize",
        "pageViewPortSize",
        "pageSize",
        "triggerSearch",
        "text",
        "requestOpenNewlySignedFile",
        "filePath",
        "pageNumber",
        "signingStarted",
        "signingFinished",
        "copyTextSelection",
        "PageView::TextCopyMode",
        "mode",
        "selectAll",
        "openAnnotationWindow",
        "Okular::Annotation*",
        "annotation",
        "reloadForms",
        "slotSelectPage",
        "slotAction",
        "Okular::Action*",
        "action",
        "slotMouseAction",
        "Okular::FormField*",
        "form",
        "Okular::Document::MouseEventType",
        "fieldMouseEventType",
        "slotFormChanged",
        "externalKeyPressEvent",
        "QKeyEvent*",
        "e",
        "finishSigning",
        "PageView::FinishSigningResult",
        "slotRealNotifyViewportChanged",
        "smoothMove",
        "slotRelayoutPages",
        "delayedResizeEvent",
        "slotRequestVisiblePixmaps",
        "newValue",
        "slotAutoScroll",
        "slotDragScroll",
        "slotShowWelcome",
        "slotShowSizeAllCursor",
        "slotHandleWebShortcutAction",
        "slotConfigureWebShortcuts",
        "slotZoom",
        "slotZoomIn",
        "slotZoomOut",
        "slotZoomActual",
        "slotFitToWidthToggled",
        "slotFitToPageToggled",
        "slotAutoFitToggled",
        "slotViewMode",
        "QAction*",
        "slotContinuousToggled",
        "slotReadingDirectionToggled",
        "leftToRight",
        "slotUpdateReadingDirectionAction",
        "slotSetMouseNormal",
        "slotSetMouseZoom",
        "slotSetMouseMagnifier",
        "slotSetMouseSelect",
        "slotSetMouseTextSelect",
        "slotSetMouseTableSelect",
        "slotSignature",
        "slotAutoScrollUp",
        "slotAutoScrollDown",
        "slotScrollUp",
        "nSteps",
        "slotScrollDown",
        "slotRotateClockwise",
        "slotRotateCounterClockwise",
        "slotRotateOriginal",
        "slotTrimMarginsToggled",
        "slotTrimToSelectionToggled",
        "slotToggleForms",
        "slotRefreshPage",
        "slotSpeakDocument",
        "slotSpeakFromCurrentPage",
        "slotSpeakCurrentPage",
        "slotStopSpeaks",
        "slotPauseResumeSpeech",
        "slotAnnotationWindowDestroyed",
        "window",
        "slotProcessMovieAction",
        "const Okular::MovieAction*",
        "slotProcessRenditionAction",
        "const Okular::RenditionAction*",
        "slotFitWindowToPage"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'rightClick'
        QtMocHelpers::SignalData<void(const Okular::Page *, const QPoint)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { 0x80000000 | 4, 2 },
        }}),
        // Signal 'mouseBackButtonClick'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mouseForwardButtonClick'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'escPressed'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fitWindowToPage'
        QtMocHelpers::SignalData<void(const QSize, const QSize)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 9, 11 },
        }}),
        // Signal 'triggerSearch'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'requestOpenNewlySignedFile'
        QtMocHelpers::SignalData<void(const QString &, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 16 },
        }}),
        // Signal 'signingStarted'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'signingFinished'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyTextSelection'
        QtMocHelpers::SlotData<void(PageView::TextCopyMode) const>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'copyTextSelection'
        QtMocHelpers::SlotData<void() const>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openAnnotationWindow'
        QtMocHelpers::SlotData<void(Okular::Annotation *, int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'reloadForms'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSelectPage'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotAction'
        QtMocHelpers::SlotData<void(Okular::Action *)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 },
        }}),
        // Slot 'slotMouseAction'
        QtMocHelpers::SlotData<void(Okular::Action *, Okular::FormField *, Okular::Document::MouseEventType)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 29, 30 }, { 0x80000000 | 32, 33 }, { 0x80000000 | 34, 35 },
        }}),
        // Slot 'slotFormChanged'
        QtMocHelpers::SlotData<void(int)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Slot 'externalKeyPressEvent'
        QtMocHelpers::SlotData<void(QKeyEvent *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'finishSigning'
        QtMocHelpers::SlotData<PageView::FinishSigningResult()>(40, 2, QMC::AccessPublic, 0x80000000 | 41),
        // Slot 'slotRealNotifyViewportChanged'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 43 },
        }}),
        // Slot 'slotRelayoutPages'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'delayedResizeEvent'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRequestVisiblePixmaps'
        QtMocHelpers::SlotData<void(int)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 47 },
        }}),
        // Slot 'slotRequestVisiblePixmaps'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotAutoScroll'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDragScroll'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowWelcome'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowSizeAllCursor'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotHandleWebShortcutAction'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotConfigureWebShortcuts'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotZoom'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotZoomIn'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotZoomOut'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotZoomActual'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotFitToWidthToggled'
        QtMocHelpers::SlotData<void(bool)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotFitToPageToggled'
        QtMocHelpers::SlotData<void(bool)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotAutoFitToggled'
        QtMocHelpers::SlotData<void(bool)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotViewMode'
        QtMocHelpers::SlotData<void(QAction *)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 62, 30 },
        }}),
        // Slot 'slotContinuousToggled'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotReadingDirectionToggled'
        QtMocHelpers::SlotData<void(bool)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 65 },
        }}),
        // Slot 'slotUpdateReadingDirectionAction'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetMouseNormal'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetMouseZoom'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetMouseMagnifier'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetMouseSelect'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetMouseTextSelect'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSetMouseTableSelect'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSignature'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAutoScrollUp'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAutoScrollDown'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotScrollUp'
        QtMocHelpers::SlotData<void(int)>(76, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 77 },
        }}),
        // Slot 'slotScrollUp'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotScrollDown'
        QtMocHelpers::SlotData<void(int)>(78, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 77 },
        }}),
        // Slot 'slotScrollDown'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotRotateClockwise'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRotateCounterClockwise'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRotateOriginal'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotTrimMarginsToggled'
        QtMocHelpers::SlotData<void(bool)>(82, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotTrimToSelectionToggled'
        QtMocHelpers::SlotData<void(bool)>(83, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'slotToggleForms'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotRefreshPage'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSpeakDocument'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSpeakFromCurrentPage'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSpeakCurrentPage'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotStopSpeaks'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotPauseResumeSpeech'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotAnnotationWindowDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(91, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 92 },
        }}),
        // Slot 'slotProcessMovieAction'
        QtMocHelpers::SlotData<void(const Okular::MovieAction *)>(93, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 94, 30 },
        }}),
        // Slot 'slotProcessRenditionAction'
        QtMocHelpers::SlotData<void(const Okular::RenditionAction *)>(95, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 96, 30 },
        }}),
        // Slot 'slotFitWindowToPage'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PageView, qt_meta_tag_ZN8PageViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PageView::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractScrollArea::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8PageViewE_t>.metaTypes,
    nullptr
} };

void PageView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PageView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->rightClick((*reinterpret_cast<std::add_pointer_t<const Okular::Page*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 1: _t->mouseBackButtonClick(); break;
        case 2: _t->mouseForwardButtonClick(); break;
        case 3: _t->escPressed(); break;
        case 4: _t->fitWindowToPage((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2]))); break;
        case 5: _t->triggerSearch((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->requestOpenNewlySignedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 7: _t->signingStarted(); break;
        case 8: _t->signingFinished(); break;
        case 9: _t->copyTextSelection((*reinterpret_cast<std::add_pointer_t<PageView::TextCopyMode>>(_a[1]))); break;
        case 10: _t->copyTextSelection(); break;
        case 11: _t->selectAll(); break;
        case 12: _t->openAnnotationWindow((*reinterpret_cast<std::add_pointer_t<Okular::Annotation*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->reloadForms(); break;
        case 14: _t->slotSelectPage(); break;
        case 15: _t->slotAction((*reinterpret_cast<std::add_pointer_t<Okular::Action*>>(_a[1]))); break;
        case 16: _t->slotMouseAction((*reinterpret_cast<std::add_pointer_t<Okular::Action*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Okular::FormField*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Okular::Document::MouseEventType>>(_a[3]))); break;
        case 17: _t->slotFormChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->externalKeyPressEvent((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 19: { PageView::FinishSigningResult _r = _t->finishSigning();
            if (_a[0]) *reinterpret_cast<PageView::FinishSigningResult*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->slotRealNotifyViewportChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->slotRelayoutPages(); break;
        case 22: _t->delayedResizeEvent(); break;
        case 23: _t->slotRequestVisiblePixmaps((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->slotRequestVisiblePixmaps(); break;
        case 25: _t->slotAutoScroll(); break;
        case 26: _t->slotDragScroll(); break;
        case 27: _t->slotShowWelcome(); break;
        case 28: _t->slotShowSizeAllCursor(); break;
        case 29: _t->slotHandleWebShortcutAction(); break;
        case 30: _t->slotConfigureWebShortcuts(); break;
        case 31: _t->slotZoom(); break;
        case 32: _t->slotZoomIn(); break;
        case 33: _t->slotZoomOut(); break;
        case 34: _t->slotZoomActual(); break;
        case 35: _t->slotFitToWidthToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->slotFitToPageToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->slotAutoFitToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->slotViewMode((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 39: _t->slotContinuousToggled(); break;
        case 40: _t->slotReadingDirectionToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->slotUpdateReadingDirectionAction(); break;
        case 42: _t->slotSetMouseNormal(); break;
        case 43: _t->slotSetMouseZoom(); break;
        case 44: _t->slotSetMouseMagnifier(); break;
        case 45: _t->slotSetMouseSelect(); break;
        case 46: _t->slotSetMouseTextSelect(); break;
        case 47: _t->slotSetMouseTableSelect(); break;
        case 48: _t->slotSignature(); break;
        case 49: _t->slotAutoScrollUp(); break;
        case 50: _t->slotAutoScrollDown(); break;
        case 51: _t->slotScrollUp((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 52: _t->slotScrollUp(); break;
        case 53: _t->slotScrollDown((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 54: _t->slotScrollDown(); break;
        case 55: _t->slotRotateClockwise(); break;
        case 56: _t->slotRotateCounterClockwise(); break;
        case 57: _t->slotRotateOriginal(); break;
        case 58: _t->slotTrimMarginsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 59: _t->slotTrimToSelectionToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 60: _t->slotToggleForms(); break;
        case 61: _t->slotRefreshPage(); break;
        case 62: _t->slotSpeakDocument(); break;
        case 63: _t->slotSpeakFromCurrentPage(); break;
        case 64: _t->slotSpeakCurrentPage(); break;
        case 65: _t->slotStopSpeaks(); break;
        case 66: _t->slotPauseResumeSpeech(); break;
        case 67: _t->slotAnnotationWindowDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        case 68: _t->slotProcessMovieAction((*reinterpret_cast<std::add_pointer_t<const Okular::MovieAction*>>(_a[1]))); break;
        case 69: _t->slotProcessRenditionAction((*reinterpret_cast<std::add_pointer_t<const Okular::RenditionAction*>>(_a[1]))); break;
        case 70: _t->slotFitWindowToPage(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 38:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PageView::*)(const Okular::Page * , const QPoint )>(_a, &PageView::rightClick, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)()>(_a, &PageView::mouseBackButtonClick, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)()>(_a, &PageView::mouseForwardButtonClick, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)()>(_a, &PageView::escPressed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)(const QSize , const QSize )>(_a, &PageView::fitWindowToPage, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)(const QString & )>(_a, &PageView::triggerSearch, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)(const QString & , int )>(_a, &PageView::requestOpenNewlySignedFile, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)()>(_a, &PageView::signingStarted, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PageView::*)()>(_a, &PageView::signingFinished, 8))
            return;
    }
}

const QMetaObject *PageView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8PageViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Okular::DocumentObserver"))
        return static_cast< Okular::DocumentObserver*>(this);
    if (!strcmp(_clname, "Okular::View"))
        return static_cast< Okular::View*>(this);
    return QAbstractScrollArea::qt_metacast(_clname);
}

int PageView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 71)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 71;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 71)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 71;
    }
    return _id;
}

// SIGNAL 0
void PageView::rightClick(const Okular::Page * _t1, const QPoint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void PageView::mouseBackButtonClick()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PageView::mouseForwardButtonClick()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PageView::escPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PageView::fitWindowToPage(const QSize _t1, const QSize _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void PageView::triggerSearch(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void PageView::requestOpenNewlySignedFile(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void PageView::signingStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void PageView::signingFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
