/****************************************************************************
** Meta object code from reading C++ file 'js_app_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/core/script/js_app_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'js_app_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Okular5JSAppE_t {};
} // unnamed namespace

template <> constexpr inline auto Okular::JSApp::qt_create_metaobjectdata<qt_meta_tag_ZN6Okular5JSAppE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Okular::JSApp",
        "alert",
        "",
        "QJSValue",
        "arguments",
        "cMsg",
        "nIcon",
        "nType",
        "cTitle",
        "oDoc",
        "oCheckbox",
        "beep",
        "getNthPlugInName",
        "nIndex",
        "goBack",
        "goForward",
        "setInterval",
        "cExpr",
        "nMilliseconds",
        "clearInterval",
        "oInterval",
        "setTimeOut",
        "clearTimeOut",
        "oTime",
        "okular_popUpMenuEx",
        "formsVersion",
        "language",
        "numPlugIns",
        "platform",
        "plugIns",
        "printColorProfiles",
        "printerNames",
        "viewerType",
        "viewerVariation",
        "viewerVersion"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'alert'
        QtMocHelpers::MethodData<int(const QJSValue &)>(1, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'alert'
        QtMocHelpers::MethodData<int(const QString &, int, int, const QString &, const QJSValue &, const QJSValue &)>(1, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 5 }, { QMetaType::Int, 6 }, { QMetaType::Int, 7 }, { QMetaType::QString, 8 },
            { 0x80000000 | 3, 9 }, { 0x80000000 | 3, 10 },
        }}),
        // Method 'alert'
        QtMocHelpers::MethodData<int(const QString &, int, int, const QString &, const QJSValue &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 5 }, { QMetaType::Int, 6 }, { QMetaType::Int, 7 }, { QMetaType::QString, 8 },
            { 0x80000000 | 3, 9 },
        }}),
        // Method 'alert'
        QtMocHelpers::MethodData<int(const QString &, int, int, const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 5 }, { QMetaType::Int, 6 }, { QMetaType::Int, 7 }, { QMetaType::QString, 8 },
        }}),
        // Method 'alert'
        QtMocHelpers::MethodData<int(const QString &, int, int)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 5 }, { QMetaType::Int, 6 }, { QMetaType::Int, 7 },
        }}),
        // Method 'alert'
        QtMocHelpers::MethodData<int(const QString &, int)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 5 }, { QMetaType::Int, 6 },
        }}),
        // Method 'alert'
        QtMocHelpers::MethodData<int(const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 5 },
        }}),
        // Method 'beep'
        QtMocHelpers::MethodData<void(int)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Method 'beep'
        QtMocHelpers::MethodData<void()>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'getNthPlugInName'
        QtMocHelpers::MethodData<QJSValue(int) const>(12, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::Int, 13 },
        }}),
        // Method 'goBack'
        QtMocHelpers::MethodData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'goForward'
        QtMocHelpers::MethodData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'setInterval'
        QtMocHelpers::MethodData<QJSValue(const QString &, int)>(16, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 17 }, { QMetaType::Int, 18 },
        }}),
        // Method 'clearInterval'
        QtMocHelpers::MethodData<void(const QJSValue &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 20 },
        }}),
        // Method 'setTimeOut'
        QtMocHelpers::MethodData<QJSValue(const QString &, int)>(21, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { QMetaType::QString, 17 }, { QMetaType::Int, 18 },
        }}),
        // Method 'clearTimeOut'
        QtMocHelpers::MethodData<void(const QJSValue &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 23 },
        }}),
        // Method 'okular_popUpMenuEx'
        QtMocHelpers::MethodData<QJSValue(const QJSValue &)>(24, 2, QMC::AccessPublic, 0x80000000 | 3, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'formsVersion'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'language'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numPlugIns'
        QtMocHelpers::PropertyData<int>(27, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'platform'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'plugIns'
        QtMocHelpers::PropertyData<QJSValue>(29, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'printColorProfiles'
        QtMocHelpers::PropertyData<QStringList>(30, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'printerNames'
        QtMocHelpers::PropertyData<QStringList>(31, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewerType'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewerVariation'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'viewerVersion'
        QtMocHelpers::PropertyData<int>(34, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JSApp, qt_meta_tag_ZN6Okular5JSAppE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Okular::JSApp::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular5JSAppE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular5JSAppE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Okular5JSAppE_t>.metaTypes,
    nullptr
} };

void Okular::JSApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JSApp *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { int _r = _t->alert((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 1: { int _r = _t->alert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[6])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->alert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[5])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 3: { int _r = _t->alert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 4: { int _r = _t->alert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->alert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 6: { int _r = _t->alert((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->beep((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->beep(); break;
        case 9: { QJSValue _r = _t->getNthPlugInName((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 10: _t->goBack(); break;
        case 11: _t->goForward(); break;
        case 12: { QJSValue _r = _t->setInterval((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->clearInterval((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1]))); break;
        case 14: { QJSValue _r = _t->setTimeOut((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->clearTimeOut((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1]))); break;
        case 16: { QJSValue _r = _t->okular_popUpMenuEx((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 5:
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 4:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->formsVersion(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->language(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->numPlugIns(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->platform(); break;
        case 4: *reinterpret_cast<QJSValue*>(_v) = _t->plugIns(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->printColorProfiles(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->printerNames(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->viewerType(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->viewerVariation(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->viewerVersion(); break;
        default: break;
        }
    }
}

const QMetaObject *Okular::JSApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Okular::JSApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular5JSAppE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Okular::JSApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
