/****************************************************************************
** Meta object code from reading C++ file 'js_document_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/core/script/js_document_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'js_document_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Okular10JSDocumentE_t {};
} // unnamed namespace

template <> constexpr inline auto Okular::JSDocument::qt_create_metaobjectdata<qt_meta_tag_ZN6Okular10JSDocumentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Okular::JSDocument",
        "getField",
        "QJSValue",
        "",
        "cName",
        "getPageLabel",
        "nPage",
        "getPageRotation",
        "gotoNamedDest",
        "syncAnnotScan",
        "getNthFieldName",
        "nIndex",
        "getOCGs",
        "numPages",
        "pageNum",
        "documentFileName",
        "filesize",
        "path",
        "URL",
        "permStatusReady",
        "dataObjects",
        "external",
        "numFields",
        "info",
        "author",
        "creator",
        "keywords",
        "producer",
        "title",
        "subject"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'getField'
        QtMocHelpers::MethodData<QJSValue(const QString &) const>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 4 },
        }}),
        // Method 'getPageLabel'
        QtMocHelpers::MethodData<QString(int) const>(5, 3, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 6 },
        }}),
        // Method 'getPageRotation'
        QtMocHelpers::MethodData<int(int) const>(7, 3, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 6 },
        }}),
        // Method 'gotoNamedDest'
        QtMocHelpers::MethodData<void(const QString &) const>(8, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 4 },
        }}),
        // Method 'syncAnnotScan'
        QtMocHelpers::MethodData<void() const>(9, 3, QMC::AccessPublic, QMetaType::Void),
        // Method 'getNthFieldName'
        QtMocHelpers::MethodData<QJSValue(int) const>(10, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::Int, 11 },
        }}),
        // Method 'getOCGs'
        QtMocHelpers::MethodData<QJSValue(int) const>(12, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::Int, 6 },
        }}),
        // Method 'getOCGs'
        QtMocHelpers::MethodData<QJSValue() const>(12, 3, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 2),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'numPages'
        QtMocHelpers::PropertyData<int>(13, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'pageNum'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'documentFileName'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'filesize'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'path'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'URL'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'permStatusReady'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'dataObjects'
        QtMocHelpers::PropertyData<QJSValue>(20, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'external'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'numFields'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'info'
        QtMocHelpers::PropertyData<QJSValue>(23, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'author'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'creator'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'keywords'
        QtMocHelpers::PropertyData<QString>(26, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'producer'
        QtMocHelpers::PropertyData<QString>(27, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(28, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'subject'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<JSDocument, qt_meta_tag_ZN6Okular10JSDocumentE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Okular::JSDocument::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular10JSDocumentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular10JSDocumentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Okular10JSDocumentE_t>.metaTypes,
    nullptr
} };

void Okular::JSDocument::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JSDocument *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QJSValue _r = _t->getField((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QString _r = _t->getPageLabel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 2: { int _r = _t->getPageRotation((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 3: _t->gotoNamedDest((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->syncAnnotScan(); break;
        case 5: { QJSValue _r = _t->getNthFieldName((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 6: { QJSValue _r = _t->getOCGs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 7: { QJSValue _r = _t->getOCGs();
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QJSValue >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->numPages(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->pageNum(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->documentFileName(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->filesize(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->path(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->URL(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->permStatusReady(); break;
        case 7: *reinterpret_cast<QJSValue*>(_v) = _t->dataObjects(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->external(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->numFields(); break;
        case 10: *reinterpret_cast<QJSValue*>(_v) = _t->info(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->author(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->creator(); break;
        case 13: *reinterpret_cast<QString*>(_v) = _t->keywords(); break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->producer(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->subject(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setPageNum(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Okular::JSDocument::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Okular::JSDocument::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular10JSDocumentE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Okular::JSDocument::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
