/****************************************************************************
** Meta object code from reading C++ file 'dviRenderer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../okular-26.08.0/generators/dvi/dviRenderer.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dviRenderer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11dviRendererE_t {};
} // unnamed namespace

template <> constexpr inline auto dviRenderer::qt_create_metaobjectdata<qt_meta_tag_ZN11dviRendererE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dviRenderer",
        "error",
        "",
        "message",
        "duration",
        "warning",
        "notice",
        "exportPS",
        "fname",
        "options",
        "QPrinter*",
        "printer",
        "QPageLayout::Orientation",
        "orientation",
        "handleSRCLink",
        "linkText",
        "QPoint",
        "point",
        "DocumentWidget*",
        "widget",
        "embedPostScript",
        "drawPage",
        "dviPageInfo*",
        "page",
        "getText",
        "sizeOfPage",
        "SimplePageSize",
        "PageNumber"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QString &, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'warning'
        QtMocHelpers::SignalData<void(const QString &, int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'notice'
        QtMocHelpers::SignalData<void(const QString &, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::Int, 4 },
        }}),
        // Slot 'exportPS'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &, QPrinter *, QPageLayout::Orientation)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QStringList, 9 }, { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'exportPS'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &, QPrinter *)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QStringList, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'exportPS'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QStringList, 9 },
        }}),
        // Slot 'exportPS'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'exportPS'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'handleSRCLink'
        QtMocHelpers::SlotData<void(const QString &, const QPoint, DocumentWidget *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { 0x80000000 | 16, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'embedPostScript'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'drawPage'
        QtMocHelpers::SlotData<void(dviPageInfo *)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'getText'
        QtMocHelpers::SlotData<void(dviPageInfo *)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Slot 'sizeOfPage'
        QtMocHelpers::SlotData<SimplePageSize(const PageNumber)>(25, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { 0x80000000 | 27, 23 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<dviRenderer, qt_meta_tag_ZN11dviRendererE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dviRenderer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11dviRendererE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11dviRendererE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11dviRendererE_t>.metaTypes,
    nullptr
} };

void dviRenderer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<dviRenderer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->warning((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 2: _t->notice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->exportPS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QPageLayout::Orientation>>(_a[4]))); break;
        case 4: _t->exportPS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPrinter*>>(_a[3]))); break;
        case 5: _t->exportPS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 6: _t->exportPS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->exportPS(); break;
        case 8: _t->handleSRCLink((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<DocumentWidget*>>(_a[3]))); break;
        case 9: _t->embedPostScript(); break;
        case 10: _t->drawPage((*reinterpret_cast<std::add_pointer_t<dviPageInfo*>>(_a[1]))); break;
        case 11: _t->getText((*reinterpret_cast<std::add_pointer_t<dviPageInfo*>>(_a[1]))); break;
        case 12: { SimplePageSize _r = _t->sizeOfPage((*reinterpret_cast<std::add_pointer_t<PageNumber>>(_a[1])));
            if (_a[0]) *reinterpret_cast<SimplePageSize*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QPageLayout::Orientation >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (dviRenderer::*)(const QString & , int )>(_a, &dviRenderer::error, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (dviRenderer::*)(const QString & , int )>(_a, &dviRenderer::warning, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (dviRenderer::*)(const QString & , int )>(_a, &dviRenderer::notice, 2))
            return;
    }
}

const QMetaObject *dviRenderer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dviRenderer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11dviRendererE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "bigEndianByteReader"))
        return static_cast< bigEndianByteReader*>(this);
    return QObject::qt_metacast(_clname);
}

int dviRenderer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void dviRenderer::error(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void dviRenderer::warning(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void dviRenderer::notice(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}
QT_WARNING_POP
