/****************************************************************************
** Meta object code from reading C++ file 'certificatemodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/gui/certificatemodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'certificatemodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16CertificateModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CertificateModel::qt_create_metaobjectdata<qt_meta_tag_ZN16CertificateModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CertificateModel",
        "propertyVisibleValue",
        "",
        "CertificateModel::Property",
        "p",
        "exportCertificateTo",
        "path",
        "Property",
        "Version",
        "SerialNumber",
        "Issuer",
        "IssuedOn",
        "ExpiresOn",
        "Subject",
        "PublicKey",
        "KeyUsage",
        "IssuerName",
        "IssuerEmail",
        "IssuerOrganization",
        "SubjectName",
        "SubjectEmail",
        "SubjectOrganization",
        "Sha1",
        "Sha256"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'propertyVisibleValue'
        QtMocHelpers::MethodData<QString(CertificateModel::Property) const>(1, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'exportCertificateTo'
        QtMocHelpers::MethodData<bool(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Property'
        QtMocHelpers::EnumData<enum Property>(7, 7, QMC::EnumFlags{}).add({
            {    8, Property::Version },
            {    9, Property::SerialNumber },
            {   10, Property::Issuer },
            {   11, Property::IssuedOn },
            {   12, Property::ExpiresOn },
            {   13, Property::Subject },
            {   14, Property::PublicKey },
            {   15, Property::KeyUsage },
            {   16, Property::IssuerName },
            {   17, Property::IssuerEmail },
            {   18, Property::IssuerOrganization },
            {   19, Property::SubjectName },
            {   20, Property::SubjectEmail },
            {   21, Property::SubjectOrganization },
            {   22, Property::Sha1 },
            {   23, Property::Sha256 },
        }),
    };
    return QtMocHelpers::metaObjectData<CertificateModel, qt_meta_tag_ZN16CertificateModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CertificateModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractTableModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CertificateModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CertificateModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16CertificateModelE_t>.metaTypes,
    nullptr
} };

void CertificateModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CertificateModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QString _r = _t->propertyVisibleValue((*reinterpret_cast<std::add_pointer_t<CertificateModel::Property>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->exportCertificateTo((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *CertificateModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CertificateModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CertificateModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractTableModel::qt_metacast(_clname);
}

int CertificateModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractTableModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
