/********************************************************************************
** Form generated from reading UI file 'welcomescreen.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WELCOMESCREEN_H
#define UI_WELCOMESCREEN_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_WelcomeScreen
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *verticalSpacer_5;
    QWidget *horizontalWidget_2;
    QHBoxLayout *horizontalLayout_2;
    QFrame *verticalFrame;
    QVBoxLayout *verticalLayout_5;
    QLabel *label;
    QSpacerItem *verticalSpacer_2;
    QLabel *appIcon;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *openButton;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *horizontalSpacer_4;
    QWidget *verticalWidget;
    QVBoxLayout *verticalLayout;
    QFrame *line_2;
    QSpacerItem *horizontalSpacer_3;
    QWidget *recentsArea;
    QVBoxLayout *verticalLayout_3;
    QWidget *horizontalWidget;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QToolButton *forgetAllButton;
    QListView *recentsListView;
    QGridLayout *gridLayout_2;
    QSpacerItem *horizontalSpacer_5;
    QPushButton *closeButton;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *verticalSpacer_6;

    void setupUi(QWidget *WelcomeScreen)
    {
        if (WelcomeScreen->objectName().isEmpty())
            WelcomeScreen->setObjectName("WelcomeScreen");
        WelcomeScreen->resize(1423, 850);
        WelcomeScreen->setAcceptDrops(true);
        gridLayout = new QGridLayout(WelcomeScreen);
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_8 = new QSpacerItem(48, 20, QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_8, 1, 0, 3, 1);

        verticalSpacer_5 = new QSpacerItem(20, 48, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout->addItem(verticalSpacer_5, 0, 1, 1, 1);

        horizontalWidget_2 = new QWidget(WelcomeScreen);
        horizontalWidget_2->setObjectName("horizontalWidget_2");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(horizontalWidget_2->sizePolicy().hasHeightForWidth());
        horizontalWidget_2->setSizePolicy(sizePolicy);
        horizontalWidget_2->setMinimumSize(QSize(0, 256));
        horizontalWidget_2->setMaximumSize(QSize(700, 16777215));
        horizontalLayout_2 = new QHBoxLayout(horizontalWidget_2);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalFrame = new QFrame(horizontalWidget_2);
        verticalFrame->setObjectName("verticalFrame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(verticalFrame->sizePolicy().hasHeightForWidth());
        verticalFrame->setSizePolicy(sizePolicy1);
        verticalFrame->setMaximumSize(QSize(192, 16777215));
        verticalFrame->setFrameShape(QFrame::NoFrame);
        verticalLayout_5 = new QVBoxLayout(verticalFrame);
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(verticalFrame);
        label->setObjectName("label");
        label->setMinimumSize(QSize(0, 40));
        QFont font;
        font.setPointSize(12);
        font.setBold(true);
        label->setFont(font);
        label->setAlignment(Qt::AlignCenter);
        label->setWordWrap(true);
        label->setMargin(0);

        verticalLayout_5->addWidget(label);

        verticalSpacer_2 = new QSpacerItem(12, 12, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout_5->addItem(verticalSpacer_2);

        appIcon = new QLabel(verticalFrame);
        appIcon->setObjectName("appIcon");
        appIcon->setText(QString::fromUtf8(""));
        appIcon->setAlignment(Qt::AlignCenter);

        verticalLayout_5->addWidget(appIcon);

        verticalSpacer_3 = new QSpacerItem(20, 12, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        verticalLayout_5->addItem(verticalSpacer_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        horizontalSpacer = new QSpacerItem(48, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        openButton = new QPushButton(verticalFrame);
        openButton->setObjectName("openButton");
        QIcon icon(QIcon::fromTheme(QString::fromUtf8("document-open")));
        openButton->setIcon(icon);

        horizontalLayout->addWidget(openButton);

        horizontalSpacer_2 = new QSpacerItem(48, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout_5->addLayout(horizontalLayout);


        horizontalLayout_2->addWidget(verticalFrame);

        horizontalSpacer_4 = new QSpacerItem(48, 20, QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_4);

        verticalWidget = new QWidget(horizontalWidget_2);
        verticalWidget->setObjectName("verticalWidget");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(verticalWidget->sizePolicy().hasHeightForWidth());
        verticalWidget->setSizePolicy(sizePolicy2);
        verticalLayout = new QVBoxLayout(verticalWidget);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        line_2 = new QFrame(verticalWidget);
        line_2->setObjectName("line_2");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(line_2->sizePolicy().hasHeightForWidth());
        line_2->setSizePolicy(sizePolicy3);
        line_2->setFrameShape(QFrame::Shape::VLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout->addWidget(line_2);

        verticalLayout->setStretch(0, 1);

        horizontalLayout_2->addWidget(verticalWidget);

        horizontalSpacer_3 = new QSpacerItem(48, 20, QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        recentsArea = new QWidget(horizontalWidget_2);
        recentsArea->setObjectName("recentsArea");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(recentsArea->sizePolicy().hasHeightForWidth());
        recentsArea->setSizePolicy(sizePolicy4);
        verticalLayout_3 = new QVBoxLayout(recentsArea);
        verticalLayout_3->setSpacing(4);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        horizontalWidget = new QWidget(recentsArea);
        horizontalWidget->setObjectName("horizontalWidget");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(horizontalWidget->sizePolicy().hasHeightForWidth());
        horizontalWidget->setSizePolicy(sizePolicy5);
        horizontalWidget->setMinimumSize(QSize(0, 36));
        horizontalLayout_3 = new QHBoxLayout(horizontalWidget);
        horizontalLayout_3->setSpacing(0);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(0, -1, 0, -1);
        label_2 = new QLabel(horizontalWidget);
        label_2->setObjectName("label_2");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy6);
        QFont font1;
        font1.setPointSize(12);
        label_2->setFont(font1);
        label_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout_3->addWidget(label_2);

        forgetAllButton = new QToolButton(horizontalWidget);
        forgetAllButton->setObjectName("forgetAllButton");
        QIcon icon1(QIcon::fromTheme(QString::fromUtf8("edit-clear-history")));
        forgetAllButton->setIcon(icon1);
        forgetAllButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
        forgetAllButton->setAutoRaise(true);

        horizontalLayout_3->addWidget(forgetAllButton);


        verticalLayout_3->addWidget(horizontalWidget);

        recentsListView = new QListView(recentsArea);
        recentsListView->setObjectName("recentsListView");
        QSizePolicy sizePolicy7(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(1);
        sizePolicy7.setHeightForWidth(recentsListView->sizePolicy().hasHeightForWidth());
        recentsListView->setSizePolicy(sizePolicy7);
        recentsListView->setMinimumSize(QSize(200, 128));
        recentsListView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        recentsListView->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);

        verticalLayout_3->addWidget(recentsListView);

        verticalLayout_3->setStretch(1, 1);

        horizontalLayout_2->addWidget(recentsArea);


        gridLayout->addWidget(horizontalWidget_2, 1, 1, 2, 1);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_5, 0, 0, 1, 1);

        closeButton = new QPushButton(WelcomeScreen);
        closeButton->setObjectName("closeButton");
        QSizePolicy sizePolicy8(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy8.setHorizontalStretch(0);
        sizePolicy8.setVerticalStretch(0);
        sizePolicy8.setHeightForWidth(closeButton->sizePolicy().hasHeightForWidth());
        closeButton->setSizePolicy(sizePolicy8);
        QIcon icon2(QIcon::fromTheme(QString::fromUtf8("window-close")));
        closeButton->setIcon(icon2);
        closeButton->setFlat(true);

        gridLayout_2->addWidget(closeButton, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer, 1, 1, 1, 1);


        gridLayout->addLayout(gridLayout_2, 0, 2, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_6, 1, 2, 3, 1);

        verticalSpacer_6 = new QSpacerItem(20, 48, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);

        gridLayout->addItem(verticalSpacer_6, 3, 1, 1, 1);

        gridLayout->setRowStretch(0, 3);
        gridLayout->setRowStretch(1, 5);
        gridLayout->setRowStretch(3, 3);
        gridLayout->setColumnStretch(0, 1);
        gridLayout->setColumnStretch(1, 4);
        gridLayout->setColumnStretch(2, 1);

        retranslateUi(WelcomeScreen);

        QMetaObject::connectSlotsByName(WelcomeScreen);
    } // setupUi

    void retranslateUi(QWidget *WelcomeScreen)
    {
        label->setText(tr2i18n("Welcome to Okular", nullptr));
        openButton->setText(tr2i18n("Open Document...", "on welcome screen"));
        label_2->setText(tr2i18n("Recent documents", "on welcome screen"));
        forgetAllButton->setText(tr2i18n("Forget All", "on welcome screen (recent document list)"));
#if QT_CONFIG(tooltip)
        closeButton->setToolTip(tr2i18n("Hide welcome screen", nullptr));
#endif // QT_CONFIG(tooltip)
        closeButton->setText(QString());
        (void)WelcomeScreen;
    } // retranslateUi

};

namespace Ui {
    class WelcomeScreen: public Ui_WelcomeScreen {};
} // namespace Ui

QT_END_NAMESPACE

#endif // WELCOMESCREEN_H

