/****************************************************************************
** Meta object code from reading C++ file 'shell.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../okular-26.08.0/shell/shell.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shell.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5ShellE_t {};
} // unnamed namespace

template <> constexpr inline auto Shell::qt_create_metaobjectdata<qt_meta_tag_ZN5ShellE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Shell",
        "D-Bus Interface",
        "org.kde.okular",
        "moveSplitter",
        "",
        "sideWidgetSize",
        "tryRaise",
        "Q_NOREPLY",
        "startupId",
        "openDocument",
        "urlString",
        "serializedOptions",
        "canOpenDocs",
        "numDocs",
        "desktop",
        "fileOpen",
        "slotUpdateFullScreen",
        "slotShowMenubar",
        "openUrl",
        "QUrl",
        "url",
        "showOpenRecentMenu",
        "closeUrl",
        "print",
        "setPrintEnabled",
        "enabled",
        "setCloseEnabled",
        "setTabIcon",
        "QMimeType",
        "mimeType",
        "handleDroppedUrls",
        "QList<QUrl>",
        "urls",
        "triggerUpdateRecentItems",
        "maxItems",
        "openNewlySignedFile",
        "path",
        "pageNumber",
        "setActiveTab",
        "tab",
        "closeTab",
        "activateNextTab",
        "activatePrevTab",
        "undoCloseTab",
        "moveTabData",
        "from",
        "to",
        "slotFitWindowToPage",
        "QSize",
        "pageViewSize",
        "pageSize",
        "hideWelcomeScreen",
        "showWelcomeScreen",
        "refreshRecentsOnWelcomeScreen",
        "forgetRecentItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'moveSplitter'
        QtMocHelpers::SignalData<void(int)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'tryRaise'
        QtMocHelpers::SlotData<void(const QString &)>(6, 7, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'openDocument'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(9, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'openDocument'
        QtMocHelpers::SlotData<bool(const QString &)>(9, 4, QMC::AccessPublic | QMC::MethodCloned | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 10 },
        }}),
        // Slot 'canOpenDocs'
        QtMocHelpers::SlotData<bool(int, int)>(12, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::Int, 13 }, { QMetaType::Int, 14 },
        }}),
        // Slot 'fileOpen'
        QtMocHelpers::SlotData<void()>(15, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotUpdateFullScreen'
        QtMocHelpers::SlotData<void()>(16, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotShowMenubar'
        QtMocHelpers::SlotData<void()>(17, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(const QUrl &, const QString &)>(18, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'openUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(18, 4, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'showOpenRecentMenu'
        QtMocHelpers::SlotData<void()>(21, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'closeUrl'
        QtMocHelpers::SlotData<void()>(22, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(23, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setPrintEnabled'
        QtMocHelpers::SlotData<void(bool)>(24, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setCloseEnabled'
        QtMocHelpers::SlotData<void(bool)>(26, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Slot 'setTabIcon'
        QtMocHelpers::SlotData<void(const QMimeType &)>(27, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'handleDroppedUrls'
        QtMocHelpers::SlotData<void(const QList<QUrl> &)>(30, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'triggerUpdateRecentItems'
        QtMocHelpers::SlotData<void(const int)>(33, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 34 },
        }}),
        // Slot 'openNewlySignedFile'
        QtMocHelpers::SlotData<void(const QString &, int)>(35, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 36 }, { QMetaType::Int, 37 },
        }}),
        // Slot 'setActiveTab'
        QtMocHelpers::SlotData<void(int)>(38, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 39 },
        }}),
        // Slot 'closeTab'
        QtMocHelpers::SlotData<void(int)>(40, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 39 },
        }}),
        // Slot 'activateNextTab'
        QtMocHelpers::SlotData<void()>(41, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'activatePrevTab'
        QtMocHelpers::SlotData<void()>(42, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'undoCloseTab'
        QtMocHelpers::SlotData<void()>(43, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'moveTabData'
        QtMocHelpers::SlotData<void(int, int)>(44, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 45 }, { QMetaType::Int, 46 },
        }}),
        // Slot 'slotFitWindowToPage'
        QtMocHelpers::SlotData<void(const QSize, const QSize)>(47, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 }, { 0x80000000 | 48, 50 },
        }}),
        // Slot 'hideWelcomeScreen'
        QtMocHelpers::SlotData<void()>(51, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showWelcomeScreen'
        QtMocHelpers::SlotData<void()>(52, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'refreshRecentsOnWelcomeScreen'
        QtMocHelpers::SlotData<void()>(53, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'forgetRecentItem'
        QtMocHelpers::SlotData<void(QUrl const &)>(54, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Shell, qt_meta_tag_ZN5ShellE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Shell::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::MainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ShellE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ShellE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5ShellE_t>.metaTypes,
    nullptr
} };

void Shell::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Shell *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->moveSplitter((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->tryRaise((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: { bool _r = _t->openDocument((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->openDocument((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->canOpenDocs((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->fileOpen(); break;
        case 6: _t->slotUpdateFullScreen(); break;
        case 7: _t->slotShowMenubar(); break;
        case 8: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 9: _t->openUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 10: _t->showOpenRecentMenu(); break;
        case 11: _t->closeUrl(); break;
        case 12: _t->print(); break;
        case 13: _t->setPrintEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->setCloseEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->setTabIcon((*reinterpret_cast<std::add_pointer_t<QMimeType>>(_a[1]))); break;
        case 16: _t->handleDroppedUrls((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1]))); break;
        case 17: _t->triggerUpdateRecentItems((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->openNewlySignedFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 19: _t->setActiveTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->closeTab((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->activateNextTab(); break;
        case 22: _t->activatePrevTab(); break;
        case 23: _t->undoCloseTab(); break;
        case 24: _t->moveTabData((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 25: _t->slotFitWindowToPage((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize>>(_a[2]))); break;
        case 26: _t->hideWelcomeScreen(); break;
        case 27: _t->showWelcomeScreen(); break;
        case 28: _t->refreshRecentsOnWelcomeScreen(); break;
        case 29: _t->forgetRecentItem((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Shell::*)(int )>(_a, &Shell::moveSplitter, 0))
            return;
    }
}

const QMetaObject *Shell::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Shell::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5ShellE_t>.strings))
        return static_cast<void*>(this);
    return KParts::MainWindow::qt_metacast(_clname);
}

int Shell::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::MainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 30;
    }
    return _id;
}

// SIGNAL 0
void Shell::moveSplitter(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
