/********************************************************************************
** Form generated from reading UI file 'pdfsettingswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PDFSETTINGSWIDGET_H
#define UI_PDFSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_PDFSettingsWidgetBase
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_3;
    QComboBox *kcfg_OverprintPreviewEnabled;
    QSpacerItem *horizontalSpacer_3;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label;
    KComboBox *kcfg_EnhanceThinLines;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QComboBox *kcfg_PrintScaleMode;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *kcfg_CheckOCSPServers;
    QWidget *signatureBackendContainer;
    QHBoxLayout *horizontalLayout;
    QLabel *label_signature_backend;
    KComboBox *kcfg_SignatureBackend;
    QSpacerItem *horizontalSpacer_4;
    QCheckBox *kcfg_EnablePgp;
    QGroupBox *certDBGroupBox;
    QFormLayout *formLayout;
    QRadioButton *kcfg_UseDefaultCertDB;
    QRadioButton *customRadioButton;
    QLabel *defaultLabel;
    QGroupBox *certificatesGroup;
    QVBoxLayout *certificatesPlaceholder;
    QPushButton *loadSignaturesButton;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *PDFSettingsWidgetBase)
    {
        if (PDFSettingsWidgetBase->objectName().isEmpty())
            PDFSettingsWidgetBase->setObjectName("PDFSettingsWidgetBase");
        PDFSettingsWidgetBase->resize(721, 451);
        verticalLayout = new QVBoxLayout(PDFSettingsWidgetBase);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_3 = new QLabel(PDFSettingsWidgetBase);
        label_3->setObjectName("label_3");

        horizontalLayout_4->addWidget(label_3);

        kcfg_OverprintPreviewEnabled = new QComboBox(PDFSettingsWidgetBase);
        kcfg_OverprintPreviewEnabled->addItem(QString());
        kcfg_OverprintPreviewEnabled->addItem(QString());
        kcfg_OverprintPreviewEnabled->addItem(QString());
        kcfg_OverprintPreviewEnabled->setObjectName("kcfg_OverprintPreviewEnabled");

        horizontalLayout_4->addWidget(kcfg_OverprintPreviewEnabled);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_3);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        label = new QLabel(PDFSettingsWidgetBase);
        label->setObjectName("label");
        label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(label);

        kcfg_EnhanceThinLines = new KComboBox(PDFSettingsWidgetBase);
        kcfg_EnhanceThinLines->addItem(QString());
        kcfg_EnhanceThinLines->addItem(QString());
        kcfg_EnhanceThinLines->addItem(QString());
        kcfg_EnhanceThinLines->setObjectName("kcfg_EnhanceThinLines");

        horizontalLayout_2->addWidget(kcfg_EnhanceThinLines);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_2 = new QLabel(PDFSettingsWidgetBase);
        label_2->setObjectName("label_2");

        horizontalLayout_3->addWidget(label_2);

        kcfg_PrintScaleMode = new QComboBox(PDFSettingsWidgetBase);
        kcfg_PrintScaleMode->addItem(QString());
        kcfg_PrintScaleMode->addItem(QString());
        kcfg_PrintScaleMode->addItem(QString());
        kcfg_PrintScaleMode->setObjectName("kcfg_PrintScaleMode");

        horizontalLayout_3->addWidget(kcfg_PrintScaleMode);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_3);

        kcfg_CheckOCSPServers = new QCheckBox(PDFSettingsWidgetBase);
        kcfg_CheckOCSPServers->setObjectName("kcfg_CheckOCSPServers");

        verticalLayout->addWidget(kcfg_CheckOCSPServers);

        signatureBackendContainer = new QWidget(PDFSettingsWidgetBase);
        signatureBackendContainer->setObjectName("signatureBackendContainer");
        horizontalLayout = new QHBoxLayout(signatureBackendContainer);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        label_signature_backend = new QLabel(signatureBackendContainer);
        label_signature_backend->setObjectName("label_signature_backend");

        horizontalLayout->addWidget(label_signature_backend);

        kcfg_SignatureBackend = new KComboBox(signatureBackendContainer);
        kcfg_SignatureBackend->setObjectName("kcfg_SignatureBackend");

        horizontalLayout->addWidget(kcfg_SignatureBackend);

        horizontalSpacer_4 = new QSpacerItem(480, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);


        verticalLayout->addWidget(signatureBackendContainer);

        kcfg_EnablePgp = new QCheckBox(PDFSettingsWidgetBase);
        kcfg_EnablePgp->setObjectName("kcfg_EnablePgp");

        verticalLayout->addWidget(kcfg_EnablePgp);

        certDBGroupBox = new QGroupBox(PDFSettingsWidgetBase);
        certDBGroupBox->setObjectName("certDBGroupBox");
        formLayout = new QFormLayout(certDBGroupBox);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        kcfg_UseDefaultCertDB = new QRadioButton(certDBGroupBox);
        kcfg_UseDefaultCertDB->setObjectName("kcfg_UseDefaultCertDB");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, kcfg_UseDefaultCertDB);

        customRadioButton = new QRadioButton(certDBGroupBox);
        customRadioButton->setObjectName("customRadioButton");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, customRadioButton);

        defaultLabel = new QLabel(certDBGroupBox);
        defaultLabel->setObjectName("defaultLabel");
        defaultLabel->setText(QString::fromUtf8(""));

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, defaultLabel);


        verticalLayout->addWidget(certDBGroupBox);

        certificatesGroup = new QGroupBox(PDFSettingsWidgetBase);
        certificatesGroup->setObjectName("certificatesGroup");
        certificatesPlaceholder = new QVBoxLayout(certificatesGroup);
        certificatesPlaceholder->setObjectName("certificatesPlaceholder");

        verticalLayout->addWidget(certificatesGroup);

        loadSignaturesButton = new QPushButton(PDFSettingsWidgetBase);
        loadSignaturesButton->setObjectName("loadSignaturesButton");

        verticalLayout->addWidget(loadSignaturesButton);

        spacerItem = new QSpacerItem(20, 4, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);


        retranslateUi(PDFSettingsWidgetBase);

        QMetaObject::connectSlotsByName(PDFSettingsWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *PDFSettingsWidgetBase)
    {
        label_3->setText(tr2i18n("Enable overprint preview:", nullptr));
        kcfg_OverprintPreviewEnabled->setItemText(0, tr2i18n("Never", nullptr));
        kcfg_OverprintPreviewEnabled->setItemText(1, tr2i18n("Always", nullptr));
        kcfg_OverprintPreviewEnabled->setItemText(2, tr2i18n("Automatic", nullptr));

        label->setText(tr2i18n("Enhance thin lines:", nullptr));
        kcfg_EnhanceThinLines->setItemText(0, tr2i18n("No", nullptr));
        kcfg_EnhanceThinLines->setItemText(1, tr2i18n("Solid", nullptr));
        kcfg_EnhanceThinLines->setItemText(2, tr2i18n("Shape", nullptr));

        label_2->setText(tr2i18n("Default scaling mode when printing:", nullptr));
        kcfg_PrintScaleMode->setItemText(0, tr2i18n("Fit to printable area", nullptr));
        kcfg_PrintScaleMode->setItemText(1, tr2i18n("Fit to full page", nullptr));
        kcfg_PrintScaleMode->setItemText(2, tr2i18n("None; print original size", nullptr));

#if QT_CONFIG(tooltip)
        kcfg_CheckOCSPServers->setToolTip(tr2i18n("Enabling this option will allow Okular to contact 3rd-party OCSP servers to check if the certificates used for digital signing have been revoked since their creation.", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_CheckOCSPServers->setText(tr2i18n("Check revocation of digital signatures' certificates using 3rd-party servers", nullptr));
        label_signature_backend->setText(tr2i18n("Signature backend:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_EnablePgp->setToolTip(tr2i18n("These signatures can only be validated by modern Okular users with GnuPG backend activated", "@info:tooltip"));
#endif // QT_CONFIG(tooltip)
        kcfg_EnablePgp->setText(tr2i18n("Allow PGP keys for signatures (Okular extension)", "@option:check"));
        certDBGroupBox->setTitle(tr2i18n("Certificate Database", nullptr));
        kcfg_UseDefaultCertDB->setText(tr2i18n("Default:", nullptr));
        customRadioButton->setText(tr2i18n("Custom:", nullptr));
        certificatesGroup->setTitle(tr2i18n("Available Certificates", nullptr));
        loadSignaturesButton->setText(tr2i18n("&Load Signatures", nullptr));
        (void)PDFSettingsWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class PDFSettingsWidgetBase: public Ui_PDFSettingsWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PDFSETTINGSWIDGET_H

