/****************************************************************************
** Meta object code from reading C++ file 'textdocumentgenerator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../okular-26.08.0/core/textdocumentgenerator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'textdocumentgenerator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Okular21TextDocumentConverterE_t {};
} // unnamed namespace

template <> constexpr inline auto Okular::TextDocumentConverter::qt_create_metaobjectdata<qt_meta_tag_ZN6Okular21TextDocumentConverterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Okular::TextDocumentConverter",
        "addAction",
        "",
        "Okular::Action*",
        "link",
        "cursorBegin",
        "cursorEnd",
        "addAnnotation",
        "Okular::Annotation*",
        "annotation",
        "addTitle",
        "level",
        "title",
        "QTextBlock",
        "position",
        "addMetaData",
        "Okular::DocumentInfo::Key",
        "key",
        "value",
        "error",
        "message",
        "duration",
        "warning",
        "notice"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'addAction'
        QtMocHelpers::SignalData<void(Okular::Action *, int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'addAnnotation'
        QtMocHelpers::SignalData<void(Okular::Annotation *, int, int)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { QMetaType::Int, 5 }, { QMetaType::Int, 6 },
        }}),
        // Signal 'addTitle'
        QtMocHelpers::SignalData<void(int, const QString &, const QTextBlock &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 12 }, { 0x80000000 | 13, 14 },
        }}),
        // Signal 'addMetaData'
        QtMocHelpers::SignalData<void(Okular::DocumentInfo::Key, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QString, 18 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QString &, int)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::Int, 21 },
        }}),
        // Signal 'warning'
        QtMocHelpers::SignalData<void(const QString &, int)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::Int, 21 },
        }}),
        // Signal 'notice'
        QtMocHelpers::SignalData<void(const QString &, int)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::Int, 21 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextDocumentConverter, qt_meta_tag_ZN6Okular21TextDocumentConverterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Okular::TextDocumentConverter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular21TextDocumentConverterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular21TextDocumentConverterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Okular21TextDocumentConverterE_t>.metaTypes,
    nullptr
} };

void Okular::TextDocumentConverter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextDocumentConverter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->addAction((*reinterpret_cast<std::add_pointer_t<Okular::Action*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 1: _t->addAnnotation((*reinterpret_cast<std::add_pointer_t<Okular::Annotation*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->addTitle((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QTextBlock>>(_a[3]))); break;
        case 3: _t->addMetaData((*reinterpret_cast<std::add_pointer_t<Okular::DocumentInfo::Key>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 4: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 5: _t->warning((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->notice((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Okular::DocumentInfo::Key >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TextDocumentConverter::*)(Okular::Action * , int , int )>(_a, &TextDocumentConverter::addAction, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextDocumentConverter::*)(Okular::Annotation * , int , int )>(_a, &TextDocumentConverter::addAnnotation, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextDocumentConverter::*)(int , const QString & , const QTextBlock & )>(_a, &TextDocumentConverter::addTitle, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextDocumentConverter::*)(Okular::DocumentInfo::Key , const QString & )>(_a, &TextDocumentConverter::addMetaData, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextDocumentConverter::*)(const QString & , int )>(_a, &TextDocumentConverter::error, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextDocumentConverter::*)(const QString & , int )>(_a, &TextDocumentConverter::warning, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TextDocumentConverter::*)(const QString & , int )>(_a, &TextDocumentConverter::notice, 6))
            return;
    }
}

const QMetaObject *Okular::TextDocumentConverter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Okular::TextDocumentConverter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular21TextDocumentConverterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Okular::TextDocumentConverter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Okular::TextDocumentConverter::addAction(Okular::Action * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void Okular::TextDocumentConverter::addAnnotation(Okular::Annotation * _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void Okular::TextDocumentConverter::addTitle(int _t1, const QString & _t2, const QTextBlock & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void Okular::TextDocumentConverter::addMetaData(Okular::DocumentInfo::Key _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Okular::TextDocumentConverter::error(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void Okular::TextDocumentConverter::warning(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void Okular::TextDocumentConverter::notice(const QString & _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6Okular21TextDocumentGeneratorE_t {};
} // unnamed namespace

template <> constexpr inline auto Okular::TextDocumentGenerator::qt_create_metaobjectdata<qt_meta_tag_ZN6Okular21TextDocumentGeneratorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Okular::TextDocumentGenerator"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TextDocumentGenerator, qt_meta_tag_ZN6Okular21TextDocumentGeneratorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Okular::TextDocumentGenerator::staticMetaObject = { {
    QMetaObject::SuperData::link<Generator::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular21TextDocumentGeneratorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular21TextDocumentGeneratorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Okular21TextDocumentGeneratorE_t>.metaTypes,
    nullptr
} };

void Okular::TextDocumentGenerator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TextDocumentGenerator *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Okular::TextDocumentGenerator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Okular::TextDocumentGenerator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Okular21TextDocumentGeneratorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Okular::ConfigInterface"))
        return static_cast< Okular::ConfigInterface*>(this);
    if (!strcmp(_clname, "org.kde.okular.ConfigInterface/0.1"))
        return static_cast< Okular::ConfigInterface*>(this);
    return Generator::qt_metacast(_clname);
}

int Okular::TextDocumentGenerator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Generator::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
